SystemOrganization addCategory: #'Spec-Widgets'!
SystemOrganization addCategory: #'Spec-Widgets-PolyWidgets'!
SystemOrganization addCategory: #'Spec-Widgets-Utilities'!

PanelMorph subclass: #PanelMorphWithSplitters
	instanceVariableNames: 'paneMorphs'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets-Utilities'!

!PanelMorphWithSplitters commentStamp: '<historical>' prior: 0!
A PanelMorphWithSplitters is which add splitters the way SystemWindow do it!

!PanelMorphWithSplitters methodsFor: 'submorphs-add/remove' stamp: 'BenjaminVanRyseghem 3/23/2012 17:00'!
addMorph: aMorph

	paneMorphs add: aMorph.
	^ super addMorph: aMorph! !

!PanelMorphWithSplitters methodsFor: 'submorphs-add/remove' stamp: ''!
addMorph: aMorph fullFrame: aLayout

	paneMorphs add: aMorph.
	super addMorph: aMorph fullFrame: aLayout! !

!PanelMorphWithSplitters methodsFor: 'private' stamp: 'IgorStasenko 12/19/2012 18:13'!
addPaneHSplitterBetween: topMorphs and: bottomMorphs
	"Add a horizontal splitter for the given morphs that share a common bottom fraction.
	If there is a horizontal discontinuity apply the splitter to the first contiguous group.
	Answer the morphs to which the splitter was applied."
	
	|targetY fixed rightFraction leftFrame rightFrame sorted morph topGroup bottomGroup splitter offset|
	topMorphs ifEmpty: [^self].
	targetY := topMorphs first layoutFrame bottomFraction.
	fixed := topMorphs select: [:m | m layoutFrame topFraction = m layoutFrame bottomFraction].
		"fixed morphs appear in both top and bottom"
	sorted := ((topMorphs reject: [:m | m layoutFrame topFraction = m layoutFrame bottomFraction])
		asSortedCollection: [:a :b | a layoutFrame rightFraction = b layoutFrame rightFraction
			ifTrue: [a layoutFrame leftFraction <= b layoutFrame leftFraction]
			ifFalse: [a layoutFrame rightFraction <= b layoutFrame rightFraction]]) readStream.
	sorted contents ifEmpty: [^fixed].
	topGroup := OrderedCollection new.
	rightFraction := sorted contents first layoutFrame leftFraction.
	[sorted atEnd or: [morph := sorted next.
			(morph layoutFrame leftFraction ~= rightFraction and: [
				morph layoutFrame rightFraction ~= rightFraction])]] whileFalse: [
		topGroup add: morph.
		rightFraction := morph layoutFrame rightFraction].
	leftFrame := topGroup first layoutFrame.
	rightFrame := topGroup last layoutFrame.
	bottomGroup := (bottomMorphs 
			reject: [:m | m layoutFrame topFraction = m layoutFrame bottomFraction])
			select: [:m |
		(m layoutFrame leftFraction
			between: leftFrame leftFraction
			and: rightFrame rightFraction) or: [
		m layoutFrame rightFraction
			between: leftFrame leftFraction
			and: rightFrame rightFraction]].
	offset := (topGroup collect: [:m | m layoutFrame bottomOffset]) max.
	splitter := ProportionalSplitterMorph new beSplitsTopAndBottom.
	splitter layoutFrame: ( (leftFrame leftFraction @ targetY corner: rightFrame rightFraction @ targetY) asLayoutFrame 
								leftOffset: leftFrame leftOffset;
								rightOffset: rightFrame rightOffset;
								topOffset: offset;
								bottomOffset: 4+offset ).
	topGroup := topGroup, fixed.
	topGroup do: [:m | splitter addLeftOrTop: m].
	bottomGroup do: [:m | splitter addRightOrBottom: m].
	self addMorphBack: splitter.
	^topGroup! !

!PanelMorphWithSplitters methodsFor: 'private' stamp: ''!
addPaneHSplitters
	"Add the horizontal pane splitters."

	|remaining targetBottom sameBottom sameTop|
	remaining := paneMorphs reject: [:each |
			each layoutFrame bottomFraction = 1 or: [
				each layoutFrame bottomFraction = 0]].
	[remaining notEmpty] whileTrue: [
		targetBottom := remaining first layoutFrame bottomFraction.
		sameBottom := remaining select: [:each |
			each layoutFrame bottomFraction = targetBottom].
		sameTop := paneMorphs select: [:each |
			each layoutFrame topFraction = targetBottom].
		remaining := remaining
			copyWithoutAll: (self addPaneHSplitterBetween: sameBottom and: sameTop)]! !

!PanelMorphWithSplitters methodsFor: 'private' stamp: 'BenjaminVanRyseghem 3/23/2012 17:01'!
addPaneSplitters
	"Add the vertical and horizontal pane splitters."

	self removePaneSplitters.
	self addPaneVSplitters.
	self addPaneHSplitters.
	self linkSplittersToSplitters! !

!PanelMorphWithSplitters methodsFor: 'private' stamp: 'IgorStasenko 12/19/2012 18:13'!
addPaneVSplitterBetween: leftMorphs and: rightMorphs
	"Add a vertical splitter for the given morphs that share a common right fraction.
	If there is a vertical discontinuity apply the splitter to the first contiguous group.
	Answer the morphs to which the splitter was applied."
	
	|targetX fixed bottomFraction topFrame bottomFrame sorted morph leftGroup rightGroup splitter offset|
	leftMorphs ifEmpty: [^self].
	targetX := leftMorphs first layoutFrame rightFraction.
	fixed := leftMorphs select: [:m | m layoutFrame leftFraction = m layoutFrame rightFraction].
		"fixed morphs appear in both top and bottom"
	sorted := ((leftMorphs reject: [:m | m layoutFrame leftFraction = m layoutFrame rightFraction])
		asSortedCollection: [:a :b | a layoutFrame bottomFraction = b layoutFrame bottomFraction
			ifTrue: [a layoutFrame topFraction <= b layoutFrame topFraction]
			ifFalse: [a layoutFrame bottomFraction <= b layoutFrame bottomFraction]]) readStream.
	sorted contents ifEmpty: [^fixed].
	leftGroup := OrderedCollection new.
	bottomFraction := sorted contents first layoutFrame topFraction.
	[sorted atEnd or: [morph := sorted next.
			morph layoutFrame topFraction ~= bottomFraction and: [
				morph layoutFrame bottomFraction ~= bottomFraction]]] whileFalse: [
		leftGroup add: morph.
		bottomFraction := morph layoutFrame bottomFraction].
	topFrame := leftGroup first layoutFrame.
	bottomFrame := leftGroup last layoutFrame.
	rightGroup := (rightMorphs
			reject: [:m | m layoutFrame leftFraction = m layoutFrame rightFraction])
			select: [:m |
		m layoutFrame topFraction
			between: topFrame topFraction
			and: bottomFrame bottomFraction].
	offset := (leftGroup collect: [:m | m layoutFrame rightOffset]) max.
	splitter := ProportionalSplitterMorph new.
	splitter layoutFrame:  ((targetX @ topFrame topFraction corner: targetX @ bottomFrame bottomFraction) asLayoutFrame 
								leftOffset: offset;
								topOffset: topFrame topOffset;
								rightOffset: 4+offset;
								bottomOffset: bottomFrame bottomOffset).
	leftGroup := leftGroup, fixed.
	leftGroup do: [:m | splitter addLeftOrTop: m].
	rightGroup do: [:m | splitter addRightOrBottom: m].
	self addMorphBack: splitter.
	^leftGroup! !

!PanelMorphWithSplitters methodsFor: 'private' stamp: ''!
addPaneVSplitters
	"Add the vertical pane splitters."

	|remaining targetRight sameRight sameLeft |
	remaining := paneMorphs reject: [:each |
			each layoutFrame rightFraction = 1 or: [
				each layoutFrame rightFraction = 0]].
	[remaining notEmpty] whileTrue: [
		targetRight := remaining first layoutFrame rightFraction.
		sameRight := remaining select: [:each |
			each layoutFrame rightFraction = targetRight].
		sameLeft := paneMorphs select: [:each |
			each layoutFrame leftFraction = targetRight and: [
				each layoutFrame rightFraction ~= targetRight]].
		remaining := remaining
			copyWithoutAll: (self addPaneVSplitterBetween: sameRight and: sameLeft)]! !

!PanelMorphWithSplitters methodsFor: 'initialization' stamp: ''!
initialize
	"Initialization code for PanelMorphWithSplitter"

	super initialize.

	paneMorphs := OrderedCollection new.! !

!PanelMorphWithSplitters methodsFor: 'layout-properties' stamp: ''!
layoutFrame: aFrame

	self activate.
	super layoutFrame: aFrame! !

!PanelMorphWithSplitters methodsFor: 'private' stamp: ''!
linkSplittersToSplitters
	"The pane morphs are already linked. Cross link the splitters as appropriate."
	
	self splitters do: [:each |
		each splitsTopAndBottom
			ifTrue: [self splitters do: [:eachMorph |
					eachMorph splitsTopAndBottom ~= each splitsTopAndBottom ifTrue: [
						eachMorph layoutFrame bottomFraction = each layoutFrame topFraction
							ifTrue: [each addLeftOrTop: eachMorph].
						eachMorph layoutFrame topFraction = each layoutFrame bottomFraction
							ifTrue: [each addRightOrBottom: eachMorph]]]]
			ifFalse: [self splitters do: [:eachMorph |
					eachMorph splitsTopAndBottom ~= each splitsTopAndBottom ifTrue: [
						eachMorph layoutFrame rightFraction = each layoutFrame leftFraction
							ifTrue: [each addLeftOrTop: eachMorph].
						eachMorph layoutFrame leftFraction = each layoutFrame rightFraction
							ifTrue: [each addRightOrBottom: eachMorph]]]].
				
		each comeToFront.]! !

Object subclass: #MorphWrapper
	instanceVariableNames: 'layout morph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets-Utilities'!

!MorphWrapper commentStamp: '<historical>' prior: 0!
A MorphWrapper is used to wrap a morph with a layout in the goal to be easily added to another morph!

!MorphWrapper class methodsFor: 'instance creation' stamp: ''!
morph: morph layout: layout 

	^ self new
		layout: layout;
		morph: morph;
		yourself! !

!MorphWrapper methodsFor: 'adding' stamp: ''!
addIn: aContainer
		
	aContainer
		addMorph: morph
		fullFrame: layout! !

!MorphWrapper methodsFor: 'accessing' stamp: 'StephaneDucasse 12/21/2012 11:26'!
frame: rectangle
	
	layout := rectangle asLayoutFrame! !

!MorphWrapper methodsFor: 'accessing' stamp: ''!
fullFrame: anObject
	
	layout := anObject! !

!MorphWrapper methodsFor: 'accessing' stamp: ''!
morph: anObject
	
	morph := anObject! !

Object subclass: #RadioButtonGroup
	instanceVariableNames: 'buttons'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets-Utilities'!

!RadioButtonGroup commentStamp: '<historical>' prior: 0!
A RadioButtonGroup is a group for radio buttons which ensure that only radio button of this group is activated at the same time.!

!RadioButtonGroup methodsFor: 'protocol' stamp: ''!
addRadioButton: aButton

	buttons do: [:b | 
		aButton whenActivatedDo: [ b privateSetState: false ].
		b whenActivatedDo: [ aButton privateSetState: false ]].
		
	buttons add: aButton.
	aButton privateSetState: false! !

!RadioButtonGroup methodsFor: 'protocol' stamp: ''!
default: aButton

	aButton privateSetState: true.
	aButton takeKeyboardFocus.! !

!RadioButtonGroup methodsFor: 'initialization' stamp: ''!
initialize
	"Initialization code for RadioButtonGroup"

	super initialize.

	buttons := OrderedCollection new.! !

ComposableModel subclass: #AbstractBasicWidget
	instanceVariableNames: 'helpHolder borderWidth borderColor enabledHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!AbstractBasicWidget commentStamp: '<historical>' prior: 0!
AbstractBasicWidget is an abstract class for basic widgets!

!AbstractBasicWidget class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 6/13/2012 16:36'!
defaultSpec

	^ SpecLayout composed! !

!AbstractBasicWidget methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 05:27'!
borderColor
	<api: #inspect>
	"Return the border color"
	
	^ borderColor contents! !

!AbstractBasicWidget methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 19:31'!
borderColor: aColor
	<api: #color getter:#borderColor registration: #whenBorderColorChanged:>
	"Set the border width"
	
	borderColor contents: aColor ! !

!AbstractBasicWidget methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 05:17'!
borderWidth
	<api: #inspect>
	"Return the border width"
	
	^ borderWidth contents! !

!AbstractBasicWidget methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 17:46'!
borderWidth: anInteger
	<api: #integer min:0 max: 5 getter:#borderWidth registration: #whenBorderWidthChanged:>
	"Set the border width"
	
	borderWidth contents: anInteger! !

!AbstractBasicWidget methodsFor: 'as yet unclassified' stamp: 'BenjaminVanRyseghem 10/18/2012 15:21'!
buildWithSpec: aSpec
	"Build the widget using the spec name provided as argument"
	| widget |
	
	(self spec notNil and: [ self needRebuild not ])
		ifTrue: [ ^ self widget ]
		ifFalse: [ widget := SpecInterpreter buildWidgetFor: self withSpec: aSpec ].

	self ensureExtentFor: widget.
		
	^ widget! !

!AbstractBasicWidget methodsFor: 'as yet unclassified' stamp: 'BenjaminVanRyseghem 10/17/2012 13:14'!
buildWithSpecLayout: aSpecLayout
	"Build the widget using the spec name provided as argument"
	| widget |
	
	(self spec notNil and: [ self needRebuild not ])
		ifTrue: [ ^ self widget ]
		ifFalse: [ widget := SpecInterpreter interpretASpec: aSpecLayout model: self ].
		
	self ensureExtentFor: widget.
	
	^ widget! !

!AbstractBasicWidget methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/18/2012 16:51'!
color

	^ self widget
		ifNil: [ Color white ]
		ifNotNil: [:w | w color ]! !

!AbstractBasicWidget methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/18/2012 16:51'!
color: aColor

	^ self widget
		ifNotNil: [:w | w color: aColor ]! !

!AbstractBasicWidget methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 18:06'!
disable
	<api: #do>
	"Disable the label"
	
	self enabled: false! !

!AbstractBasicWidget methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 18:06'!
enable
	<api: #do>
	"Enable the label"
	
	self enabled: true! !

!AbstractBasicWidget methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 18:02'!
enabled
	<api: #inspect>
	^ enabledHolder contents! !

!AbstractBasicWidget methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 18:02'!
enabled: aBoolean
	<api: #boolean getter: #enabled registration: #whenEnabledChanged:>
	"Set if the button is enabled (clickable)"
	
	enabledHolder contents: aBoolean! !

!AbstractBasicWidget methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/12/2012 17:55'!
help
	<api: #inspect>
	"Return the ballon text"
	
	^ helpHolder contents! !

!AbstractBasicWidget methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 05:18'!
help: aString
	<api: #string getter:#help registration: #whenHelpChanged:>
	"Set the help string"
	
	helpHolder contents: aString! !

!AbstractBasicWidget methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 7/16/2012 18:09'!
initialize
	"Initialization code for AbstractBasicWidget"

	super initialize.

	helpHolder := nil asValueHolder.
	borderWidth := 0 asValueHolder.
	borderColor := Color transparent asValueHolder.
	enabledHolder := true asValueHolder.
	
	helpHolder whenChangedDo: [:string | 
		self widget ifNotNil: [:w | w setBalloonText: string ]].
	
	borderWidth whenChangedDo: [:int | 
		self widget ifNotNil: [:w | w borderWidth: int ]].
	
	borderColor whenChangedDo: [:color | 
		self widget ifNotNil: [:w | w borderColor: color ]].
	
	enabledHolder whenChangedDo: [:b |
		self widget ifNotNil: [:w | w enabled: b ]].! !

!AbstractBasicWidget methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/12/2012 18:10'!
initializeWidgets 
	"ignore this method since there is no composition in basic widgets"! !

!AbstractBasicWidget methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 17:36'!
whenBorderColorChanged: aBlock
	<api: #event>
	"Set a block to be performed when the brder width changed"
	
	borderColor whenChangedDo: aBlock! !

!AbstractBasicWidget methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 17:36'!
whenBorderWidthChanged: aBlock
	<api: #event>
	"Set a block to be performed when the brder width changed"
	
	borderWidth whenChangedDo: aBlock! !

!AbstractBasicWidget methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 18:03'!
whenEnabledChanged: aBlock
	<api: #event>
	"Set a block to performed when the widget is enabled or disabled"
	
	enabledHolder whenChangedDo: aBlock! !

!AbstractBasicWidget methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 17:36'!
whenHelpChanged: aBlock
	<api: #event>
	"Set a block to be performed when the help changed"
	
	helpHolder whenChangedDo: aBlock! !

AbstractBasicWidget subclass: #ButtonModel
	instanceVariableNames: 'actionHolder labelHolder stateHolder actionPerformedHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!ButtonModel commentStamp: '<historical>' prior: 0!
A ButtonComposableModel is an applicative model which handle a basic button.

self example!

!ButtonModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 7/13/2012 05:29'!
defaultSpec
	<spec>
	^ {#ButtonSpec.
	    		#on:getState:action:label:menu:. 	#model. #state. #action. #label. #nil.
			#getEnabledSelector:. 				#enabled.
			#hResizing:. 						#spaceFill.
			#vResizing:. 						#spaceFill.
			#borderWidth:.						#(model borderWidth).
			#borderColor:.						#(model borderColor).
			#setBalloonText:.					{ #model . #help}.
			#eventHandler:.					{	#EventHandler. #on:send:to:. #keyStroke.	#keyStroke:fromMorph:. #model	}}! !

!ButtonModel class methodsFor: 'example' stamp: 'BenjaminVanRyseghem 4/24/2012 03:09'!
example

	| b |
	b:= ButtonModel new.
	b openWithSpec.
	b label: 'Click me'.! !

!ButtonModel class methodsFor: 'specs' stamp: ''!
title

	^ 'Button'! !

!ButtonModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/11/2012 17:12'!
action

	self widget ifNotNil: [:m | m takeKeyboardFocus ].
	actionHolder contents value.
	" Here I set a dummy value just to make the holder raise an event "
	actionPerformedHolder contents: nil.! !

!ButtonModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/12/2012 20:21'!
action: aBlock
	<api: #block getter: #getAction registration: #whenActionChangedDo:>
	"set the block performed when the button is clicked"

	actionHolder contents: aBlock! !

!ButtonModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 18:17'!
eventStringForNextFocus
	"String describing the keystroke to perform to jump to the next widget"
	^ '<right>'! !

!ButtonModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 18:18'!
eventStringForPreviousFocus
	"String describing the keystroke to perform to jump to the previous widget"
	^ '<left>'! !

!ButtonModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/11/2012 17:16'!
getAction
	<api: #inspect>
	"get the block performed when the button is clicked"

	^ actionHolder contents! !

!ButtonModel methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 12/12/2012 15:17'!
initialize
	"Initialization code for ButtonComposableModel"

	super initialize.

	actionHolder := [] asValueHolder.
	labelHolder := '' asValueHolder.
	stateHolder := true asValueHolder.
	enabledHolder := true asValueHolder.
	helpHolder := nil asValueHolder.
	actionPerformedHolder := nil asValueHolder.
	
	labelHolder whenChangedDo: [	self changed: #label ].
	stateHolder whenChangedDo: [	self changed: #state ].
	enabledHolder whenChangedDo: [	self changed: #enabled ].! !

!ButtonModel methodsFor: 'focus' stamp: ''!
keyStroke: anEvent fromMorph: aMorph

	anEvent keyString = '<space>'
		ifTrue: [ self action ].
		
	super keyStroke: anEvent fromMorph: aMorph! !

!ButtonModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/11/2012 17:17'!
label
	<api: #inspect>
	^ labelHolder contents! !

!ButtonModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/12/2012 20:22'!
label: aString
	<api: #string getter: #label registration: #whenLabelChangedDo:>
	"set the label of the button"

	labelHolder contents: aString! !

!ButtonModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/11/2012 17:17'!
state
	<api: #inspect>
	^ stateHolder contents! !

!ButtonModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/12/2012 20:32'!
state: aBoolean
	<api: #boolean getter: #state registration: #whenStateChangedDo:>
	"set if the button is highlighted"
	
	^ stateHolder contents: aBoolean! !

!ButtonModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/12/2012 20:21'!
whenActionChangedDo: aBlock 

	actionHolder whenChangedDo: aBlock! !

!ButtonModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/11/2012 17:12'!
whenActionPerformedDo: aBlock
	<api: #event>
	"set a block to perform after that the button has been aclicked, and its action performed"

	actionPerformedHolder whenChangedDo: aBlock! !

!ButtonModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/12/2012 20:23'!
whenLabelChangedDo: aBlock
	<api: #event>
	"set a block to perform after that the button has been aclicked, and its action performed"

	labelHolder whenChangedDo: aBlock! !

!ButtonModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/12/2012 20:23'!
whenStateChangedDo: aBlock
	<api: #event>
	"set a block to perform after that the button has been aclicked, and its action performed"

	stateHolder whenChangedDo: aBlock! !

AbstractBasicWidget subclass: #CheckBoxModel
	instanceVariableNames: 'actionWhenActivatedHolder actionWhenDesactivatedHolder stateHolder labelClickableHolder labelHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!CheckBoxModel commentStamp: '<historical>' prior: 0!
A CheckboxModel is a spec model for Checkbox

You can also have a look at CheckBoxExample for a full example of how to use them with a group.
!

!CheckBoxModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 7/13/2012 05:29'!
defaultSpec
	<spec>
	^ {#CheckboxSpec.
	    		#on:selected:changeSelected:. 	#model. #state. #state:.
			#label:.								{	#model. #label		}.
			#beCheckbox.
			#hResizing:. 						#shrinkWrap.
			#vResizing:. 						#shrinkWrap.
			#setBalloonText:.					{ #model . #help}.
			#borderWidth:.						#(model borderWidth).
			#borderColor:.						#(model borderColor).
			#eventHandler:.					{	#EventHandler. #on:send:to:. #keyStroke.	#keyStroke:fromMorph:. #model	}.
			#labelClickable:.					{	#model. #labelClickable}.}! !

!CheckBoxModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 7/13/2012 02:00'!
title

	^ 'Checkbox Button'! !

!CheckBoxModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 16:56'!
activationAction: aBlock
	<api: #block getter: #activationAction registration: #whenActivationActionChanged:>
	"This method is used to set the action to perform when I am activated"
	
	actionWhenActivatedHolder contents: aBlock! !

!CheckBoxModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/12/2012 17:50'!
click
	<api: #do>
	"Simulate a click on the checkbox
	Used when the checkboc is a list item"

	self toggleState! !

!CheckBoxModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 17:00'!
desactivationAction: aBlock
	<api: #block getter: #desactivationAction registration: #whenDesactivationActionChanged:>
	"This method is used to set the action to perform when I am desactivated"
	
	actionWhenDesactivatedHolder contents: aBlock! !

!CheckBoxModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 18:17'!
eventStringForNextFocus
	"String describing the keystroke to perform to jump to the next widget"
	^ '<right>'! !

!CheckBoxModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 18:18'!
eventStringForPreviousFocus
	"String describing the keystroke to perform to jump to the previous widget"
	^ '<left>'! !

!CheckBoxModel methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/7/2012 12:28'!
initialize
	"initialization code for CheckBox"

	super initialize.
	
	actionWhenActivatedHolder := [] asValueHolder.
	actionWhenDesactivatedHolder := [] asValueHolder.
	enabledHolder := true asValueHolder.
	stateHolder := false asValueHolder.
	labelClickableHolder := true asValueHolder. 
	labelHolder := '' asValueHolder.
	stateHolder whenChangedDo: [:bool |
		bool
			ifTrue: actionWhenActivatedHolder contents
			ifFalse: actionWhenDesactivatedHolder contents.
		self changed: #state ].
	labelClickableHolder whenChangedDo: [:aBoolean |
		self widget ifNotNil: [:w | w labelClickable: aBoolean ]].
	labelHolder whenChangedDo: [:label |
		self widget ifNotNil: [:w | w label: label ]].
	! !

!CheckBoxModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/12/2012 17:50'!
label
	<api: #inspect>
	^ labelHolder contents! !

!CheckBoxModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 17:03'!
label: aString
	<api: #string getter: #label registration: #whenLabelChanged:>
	"Set the label of the checkbox"

	labelHolder contents: aString.! !

!CheckBoxModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/13/2012 02:10'!
labelClickable
	<api: #inspect>
	"Return true if the label can be clicked to select the checkbox"

	^ labelClickableHolder contents! !

!CheckBoxModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 17:03'!
labelClickable: aBoolean
	<api: #boolean getter: #labelClickable registration: #whenLabelClickableChanged:>
	"Set if the label can be clicked to select the checkbox"
	
	labelClickableHolder contents: aBoolean! !

!CheckBoxModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/16/2012 17:06'!
state
	<api: #inspect>
	"Return the current state of the checkBox"
	
	^ stateHolder contents! !

!CheckBoxModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/12/2012 17:49'!
state: aBoolean
	<api: #boolean getter: #state>
	"Set if the checkbox is activated or not"

	stateHolder contents: aBoolean! !

!CheckBoxModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/12/2012 17:51'!
toggleState
	<api: #do>
	"Toogle the current state of the checkbox"

	self state: self state not! !

!CheckBoxModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/12/2012 17:51'!
whenActivatedDo: aBlock
	<api: #event>
	"This method is used to propagate the event that I have been activated" 
	
	stateHolder whenChangedDo: [:bool | bool ifTrue: aBlock ]! !

!CheckBoxModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 16:52'!
whenActivationActionChanged: aBlock
	<api: #event>
	"A block performed when the activation action changed"
	actionWhenActivatedHolder whenChangedDo:  aBlock ! !

!CheckBoxModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/13/2012 01:40'!
whenChangedDo: aBlock
	<api: #event>
	"This method is used to propagate the event that I have changed"
	
	stateHolder whenChangedDo: aBlock! !

!CheckBoxModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/12/2012 17:51'!
whenDesactivatedDo: aBlock
	<api: #event>
	"This method is used to propagate the event that I have been desactivated"
	
	stateHolder whenChangedDo: [:bool | bool ifFalse: aBlock ]! !

!CheckBoxModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 16:59'!
whenDesactivationActionChanged: aBlock
	<api: #event>
	"A block performed when the desactivation action changed"
	actionWhenDesactivatedHolder whenChangedDo:  aBlock ! !

!CheckBoxModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 17:01'!
whenLabelChanged: aBlock
	"A block performed when the label changed"
	<api: #event>
	
	labelHolder whenChangedDo:  aBlock ! !

!CheckBoxModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 17:02'!
whenLabelClickableChanged: aBlock
	"A block performed when the label click is activated or descativated"
	<api: #event>
	
	labelClickableHolder whenChangedDo:  aBlock ! !

AbstractBasicWidget subclass: #DropListModel
	instanceVariableNames: 'listHolder selectionHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!DropListModel commentStamp: '<historical>' prior: 0!
A DropListModel is a Spec model for drop lists.

I am assume there is a little problem on an empty list, but frankly, who creates an empty drop list ?

See DropListExample for an example!

!DropListModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 7/13/2012 05:29'!
defaultSpec
	<spec>
	
	^ {#DropListSpec.
	    		#on:list:selected:changeSelected:. 	#model. #getList. #getIndex. #setIndex:.
			#hResizing:. 						#spaceFill.
			#vResizing:. 						#spaceFill.
			#borderWidth:.						#(model borderWidth).
			#borderColor:.						#(model borderColor).
			#setBalloonText:.					{ #model . #help}.
			#eventHandler:.					{	#EventHandler. #on:send:to:. #keyStroke.	#keyStroke:fromMorph:. #model	}}! !

!DropListModel class methodsFor: 'specs' stamp: ''!
title

	^ 'Drop List'! !

!DropListModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 18:17'!
eventStringForNextFocus
	"String describing the keystroke to perform to jump to the next widget"
	^ '<right>'! !

!DropListModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 18:18'!
eventStringForPreviousFocus
	"String describing the keystroke to perform to jump to the previous widget"
	^ '<left>'! !

!DropListModel methodsFor: 'morphic' stamp: 'StephaneDucasse 5/17/2012 18:12'!
getIndex

	^ selectionHolder index contents! !

!DropListModel methodsFor: 'morphic' stamp: 'StephaneDucasse 5/17/2012 18:02'!
getList

	^ listHolder contents collect: #label! !

!DropListModel methodsFor: 'initialization' stamp: 'StephaneDucasse 5/17/2012 19:32'!
initialize
	"Initialization code for DropListModel"

	super initialize.

	listHolder := {} asValueHolder.
	selectionHolder := SelectionValueHolder new.
	
	listHolder whenChangedDo: [ 
					self changed: #getList.
					self internalResetSelection. ].
				
	selectionHolder whenChangedDo: [ self changed: #getIndex ].
	self whenSelectedItemChanged: [:item | item value ]! !

!DropListModel methodsFor: 'private' stamp: 'StephaneDucasse 5/17/2012 18:05'!
internalResetSelection

	selectionHolder reset.
	self getList isEmpty not
		ifTrue: [ self setSelectedIndex: 1 ]! !

!DropListModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/12/2012 18:07'!
items: aList
	<api: #st getter: #listItems>
	"Populate the drop list with a list of DropItems"
	
	listHolder contents: aList! !

!DropListModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/12/2012 18:08'!
listItems
	<api: #inspect>
	"Return the list of DropItems used to populate the drop list"
	
	^ listHolder contents! !

!DropListModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/12/2012 20:02'!
listSize
	<api: #inspect>
	"Return the size of the list of choices"

	^ self listItems size! !

!DropListModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/12/2012 18:08'!
resetSelection
	<api: #do>
	"Reset the current selection state"

	selectionHolder reset! !

!DropListModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/12/2012 18:08'!
selectedIndex
	<api: #inspect>
	"Useless method but it provides a better and more consistent API"

	^ self getIndex! !

!DropListModel methodsFor: 'private' stamp: 'StephaneDucasse 5/17/2012 18:04'!
selectedIndexHolder

	^ selectionHolder index! !

!DropListModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/12/2012 18:08'!
selectedItem
	<api: #inspect>
	"Return the selected item"

	^ self selectedItemHolder contents! !

!DropListModel methodsFor: 'private' stamp: 'StephaneDucasse 5/17/2012 18:04'!
selectedItemHolder

	^ selectionHolder selection! !

!DropListModel methodsFor: 'morphic' stamp: 'StephaneDucasse 5/17/2012 18:04'!
setIndex: anIndex
	| item |
	
	selectionHolder index contents: anIndex.
	item := (listHolder contents at: anIndex ifAbsent: [ nil ]).
	item value.
	selectionHolder selection contents: item.
	self changed: #getIndex! !

!DropListModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 12/20/2012 16:21'!
setSelectedIndex: anIndex
	<api: #integer min:1 max:#listSize getter: #selectedIndex>
	"Force the selection of the item at index anIndex"
	| selection |

	selection := listHolder contents at: anIndex ifAbsent: [ ^ self ].
	selection value.
	selectionHolder index contents: anIndex.
	selectionHolder selection contents: selection! !

!DropListModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/12/2012 18:10'!
setSelectedItem: anItem
	<api: #st getter: #selectedItem>
	"Force the selection of the item anItem"
	| index realItem |
	
	index := self getList indexOf: anItem ifAbsent: [ ^ self ].
	realItem := (self listItems at: index) value.
	selectionHolder index contents: index.
	selectionHolder selection contents: realItem.! !

!DropListModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/12/2012 18:10'!
whenSelectedItemChanged: aBlock
	<api: #event>
	"Set a block to perform when the selected item is changed"

	selectionHolder selection whenChangedDo: aBlock! !

!DropListModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/12/2012 18:10'!
whenSelectionChanged: aBlock
	<api: #event>
	"Set a block to perform when the selection is changed"

	selectionHolder whenChangedDo: aBlock.! !

!DropListModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/12/2012 18:10'!
whenSelectionIndexChanged: aBlock
	<api: #event>
	"Set a block to perform when the selected index is changed"

	selectionHolder index whenChangedDo: aBlock! !

AbstractBasicWidget subclass: #LabelModel
	instanceVariableNames: 'textHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!LabelModel commentStamp: '<historical>' prior: 0!
A LabelModel is a spec model for Labels!

!LabelModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 7/13/2012 05:29'!
defaultSpec
	<spec>
	^ {#LabelSpec.
			#model:. 				#model.
			#getEnabledSelector:.	#enabled.
			#getTextSelector:.		#getText.
			#vResizing:.			#shrinkWrap.
			#hResizing:.			#spaceFill.
			#borderWidth:.						#(model borderWidth).
			#borderColor:.						#(model borderColor).
			#setBalloonText:.					{ #model . #help}.
			#eventHandler:.	{	#EventHandler. #on:send:to:. #keyStroke.	#keyStroke:fromMorph:. #model	}}! !

!LabelModel class methodsFor: 'specs' stamp: ''!
title

	^ 'Label Morph'! !

!LabelModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/13/2012 02:17'!
getText
	<api: #inspect>

	^ textHolder contents! !

!LabelModel methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 7/16/2012 18:07'!
initialize
	"Initialization code for LabelModel"

	super initialize.

	textHolder := '' asValueHolder.
	enabledHolder := true asValueHolder.
	textHolder whenChangedDo: [ self changed: #getText ].! !

!LabelModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:15'!
text: aText
	<api: #string getter: #getText registration: #whenTextChanged:>
	"Set the text of the label"
	
	textHolder contents: aText! !

!LabelModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/13/2012 02:16'!
whenTextChanged: aBlock
	<api: #event>
	"Set a block to performed when the text is changed"
	
	textHolder whenChangedDo: aBlock! !

AbstractBasicWidget subclass: #ListComposableModel
	instanceVariableNames: 'selectionHolder listHolder menuHolder shortcutsHolder multiSelectionHolder multiSelection allowToSelect listItemsCache filteringBlockHolder sortingBlockHolder wrapBlockHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!ListComposableModel commentStamp: '<historical>' prior: 0!
A ListComposableModel is an applicative model which handle a basic list.

| t |
t:= ListComposableModel new.
t openWithSpec.
t items: (Smalltalk allClasses).


self example

| t |
t:= ListComposableModel new.
t openWithSpec.
t sortingBlock: [:a :b| a name > b name].
t items: (Smalltalk allClasses).


| t |
t:= ListComposableModel new.
t openWithSpec.
t filteringBlock: [:col | col select: [:each | each name beginsWith: 'Zn']].
t sortingBlock: [:a :b| a name > b name].
t items: (Smalltalk allClasses).
!

ListComposableModel subclass: #IconListModel
	instanceVariableNames: 'iconHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!IconListModel commentStamp: '<historical>' prior: 0!
An IconListModel is a spec model for icon list!

!IconListModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 7/31/2012 17:05'!
defaultSpec
	<spec>
	^ {#IconListSpec.
	    		#model:. 						#model.
			#getListSizeSelector:. 			#listSize.
			#getListElementSelector:. 		#listElementAt:.
			#getIndexSelector:. 			#getIndex.
			#setIndexSelector:. 			#setIndex:.
			#getSelectionListSelector:.		#getSelectionStateFor:.
			#setSelectionListSelector:.		#setSelectionStateFor:at:.
			#getIconSelector:.				#getIconFor:.
			#resetListSelector:.				#resetListSelection.
			#getMenuSelector:.			#menu:shifted:.
			#keystrokeSelector:.			#keyStroke:fromMorph:.
			#setMultipleSelection:.			{#model. #multiSelection}.
			#wrapSelector:. 				#wrapItem:index:.
			#borderWidth:.						#(model borderWidth).
			#borderColor:.						#(model borderColor).
			#setBalloonText:.					{ #model . #help}.
			#hResizing:. 					#spaceFill.
			#vResizing:. 					#spaceFill}! !

!IconListModel class methodsFor: 'example' stamp: 'BenjaminVanRyseghem 7/10/2012 15:50'!
example

	self new
		icons: [:e | UITheme current forwardIcon ];
		items: (1 to: 10) asArray;
		openWithSpec! !

!IconListModel methodsFor: 'morphic' stamp: ''!
getIconFor: anItem

	^ iconHolder contents cull: anItem cull: self! !

!IconListModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:51'!
icons
	<api: #inspect>
	"Return the block which takes an item as argument and returns the icon to display in the list"
	
	^ iconHolder contents! !

!IconListModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:50'!
icons: aBlock
	<api: #block getter: #icon registration: #whenIconsChanged: >
	"Set a block which takes an item as argument and returns the icon to display in the list"
	iconHolder contents: aBlock! !

!IconListModel methodsFor: 'initialization' stamp: ''!
initialize
	"Initialization code for IconListModel"

	super initialize.

	iconHolder := [:item | nil ] asValueHolder.! !

!IconListModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/13/2012 02:51'!
whenIconsChanged: aBlock 
	<api: #block>
	
	iconHolder whenChangedDo: aBlock 

! !

!ListComposableModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 10/17/2012 13:24'!
defaultSpec
	<spec: #default>
	^ {#ListSpec.
	    		#model:. 					#model.
			#getListSizeSelector:. 			#listSize.
			#getIndexSelector:. 			#getIndex.
			#setIndexSelector:. 			#setIndex:.
			#getSelectionListSelector:.		#getSelectionStateFor:.
			#setSelectionListSelector:.		#setSelectionStateFor:at:.
			#getListElementSelector:. 		#listElementAt:.
			#resetListSelector:.			#resetListSelection.
			#getMenuSelector:.			#menu:shifted:.
			#keystrokeSelector:.			#keyStroke:fromMorph:.
			#setMultipleSelection:.			{#model. #multiSelection}.
			#wrapSelector:. 				#wrapItem:index:.
			#borderWidth:.						#(model borderWidth).
			#borderColor:.						#(model borderColor).
			#setBalloonText:.					{ #model . #help}.
			#hResizing:. 					#spaceFill.
			#vResizing:. 					#spaceFill}! !

!ListComposableModel class methodsFor: 'example' stamp: 'StephaneDucasse 5/17/2012 18:02'!
example
	| instance |

	instance := self new.
	instance openWithSpec.
	instance items: (Smalltalk allClasses).! !

!ListComposableModel class methodsFor: 'specs' stamp: ''!
title

	^ 'List'! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:20'!
allowToSelect
	<api: #inspect>
	"Return whether the list items can be selected or not"

	^ allowToSelect contents! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 17:53'!
allowToSelect: aBoolean
	<api: #boolean getter: #allowToSelect registration: #whenAllowToSelectChanged:>
	"Set if the list items can be selected or not"

	allowToSelect contents: aBoolean.
	aBoolean 
		ifFalse: [ self resetSelection ].! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:21'!
beMultipleSelection
	<api: #do>
	"Make list selection multiple"

	self multiSelection: true! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:21'!
beSingleSelection
	<api: #do>
	"Make list selection single"

	self multiSelection: false! !

!ListComposableModel methodsFor: 'initialize' stamp: 'SD 4/19/2012 15:26'!
defaultFilteringBlock 
	^ [:col | col]! !

!ListComposableModel methodsFor: 'initialize' stamp: 'StephaneDucasse 4/17/2012 19:15'!
defaultSortingBlock 
	^ [:a :b | true]! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:39'!
displayBlock
	<api: #inspect>
	"Return the one argument block used to wrap your domain specific items.
	The block should return something that can be displayed in a list - like a String or a Text"
	
	^ wrapBlockHolder contents! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:39'!
displayBlock: aBlock
	<api: #block getter:#displayBlock registration: #whenDisplayBlockChanged:>
	"Set the one argument block used to wrap your domain specific items.
	The block should return something that can be displayed in a list - like a String or a Text"
	
	wrapBlockHolder contents: aBlock! !

!ListComposableModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 18:17'!
eventStringForNextFocus
	"String describing the keystroke to perform to jump to the next widget"	
	^ '<right>'! !

!ListComposableModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 18:18'!
eventStringForPreviousFocus
	"String describing the keystroke to perform to jump to the previous widget"
	^ '<left>'! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:39'!
filteringBlock
	<api: #inspect>
	"Return the filtering of the items"
	
	^ filteringBlockHolder contents! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:38'!
filteringBlock: aBlock
	<api: #block getter: #filteringBlock registration: #whenFilteringBlockChanged:>
	"To set the filtering of the items"
	
	filteringBlockHolder contents: aBlock! !

!ListComposableModel methodsFor: 'private-morphic' stamp: 'StephaneDucasse 5/17/2012 18:22'!
getIndex
	"Morphic API - The method should only be used internally for the communication between the model and the morphic widget" 
	"Return the index of the selected item"
	
	^  selectionHolder index contents! !

!ListComposableModel methodsFor: 'private-morphic' stamp: 'StephaneDucasse 5/17/2012 18:24'!
getSelectionStateFor: anIndex
	"Morphic API - The method should only be used internally for the communication between the model and the morphic widget" 
	"Return the current state of the item -if selected or not - iun a multiple selection list"
	"Answer true if the item at index _anIndex_ is selected"
	
	^ (multiSelectionHolder at: anIndex ifAbsent: [ ^ false ]) == true! !

!ListComposableModel methodsFor: 'initialize' stamp: 'StephaneDucasse 5/17/2012 18:57'!
initialize

	super initialize.

	selectionHolder := SelectionValueHolder new.
	listHolder := Array new asValueHolder.
	sortingBlockHolder := self defaultSortingBlock asValueHolder.
	filteringBlockHolder := self defaultFilteringBlock asValueHolder.
	wrapBlockHolder := [ :object | object asStringOrText ] asValueHolder.
	menuHolder := [ :menu :shifted | nil ] asValueHolder.
	shortcutsHolder := [ :event :sourceMorph |  ] asValueHolder.
	multiSelectionHolder := IdentityDictionary new asValueHolder.
	multiSelection := false asValueHolder.
	allowToSelect := true asValueHolder.

	self registerEvents! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:21'!
items: aList
	<api: #st getter: #listItems registration: #whenListChanged:>
	"Set the items of the list.
	aList is a collection of your domain specific items"

	listHolder contents: (aList sorted: sortingBlockHolder contents)! !

!ListComposableModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 18:20'!
keyStroke: anEvent fromMorph: aMorph

	"Simulate a click on the selected item when space is pressed"
	anEvent keyString = String space
		ifTrue: [| item |
			item := self selectedItem. 
			(item notNil and: [ item respondsTo: #click ])
				ifTrue: [ ^ item click ]].

	"Now check the shortcuts defined by the user"
	shortcutsHolder contents cull: anEvent cull: aMorph.

	"Finally do a super call for the focus change"
	^ super keyStroke: anEvent fromMorph: aMorph! !

!ListComposableModel methodsFor: 'private-morphic' stamp: 'StephaneDucasse 5/17/2012 18:25'!
listElementAt: anIndex
	"Morphic API - The method should only be used internally for the communication between the model and the morphic widget" 
	"Return the item at index _anIndex_"
	
	^ self listItems at: anIndex ifAbsent: [ nil ]! !

!ListComposableModel methodsFor: 'private-morphic' stamp: 'BenjaminVanRyseghem 7/16/2012 19:10'!
listElementAt: anIndex ifAbsent: aBlock
	"Morphic API - The method should only be used internally for the communication between the model and the morphic widget" 
	"Return the item at index _anIndex_"
	
	^ self listItems at: anIndex ifAbsent: aBlock! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:21'!
listItems
	<api: #inspect>
	"Return the items of the list. They are your domain specific items"
	
	^ listItemsCache ifNil: [ listItemsCache :=  filteringBlockHolder contents value: listHolder contents ]
	! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:31'!
listSize
	<api: #inspect>
	"Return the size of the list"
	
	^ self listItems size! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:33'!
menu
	<api: #inspect>
	"Return the block used to defined the menu"
	
	^ menuHolder contents! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:34'!
menu: aBlock
	<api: #block getter: #menu registration: #whenMenuChanged:>
	"Set the block used to defined the menu"
	
	menuHolder contents: aBlock! !

!ListComposableModel methodsFor: 'private-morphic' stamp: 'StephaneDucasse 5/17/2012 18:49'!
menu: aMenu shifted: aBoolean
	"Morphic API - The method should only be used internally for the communication between the model and the morphic widget" 
	"Build the menu when you right click on an item"

	^ menuHolder contents cull: aMenu cull: aBoolean! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:22'!
multiSelection
	<api: #inspect>
	"Return true if the list has a multiple selection.
	False if the list has a single selection"
	
	^ multiSelection contents! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:22'!
multiSelection: aBoolean
	<api: #boolean getter: #multiSelection registration: #whenMultiSelectionChanged:>
	"Make the list seelction become multiple if aBoolean is true.
	Otherwise set the selection as single"
	
	multiSelection contents: aBoolean.
	! !

!ListComposableModel methodsFor: 'private' stamp: 'BenjaminVanRyseghem 5/7/2012 13:23'!
refreshListItems

	listItemsCache := nil.
	^ self changed: #listElementAt:! !

!ListComposableModel methodsFor: 'initialize' stamp: 'BenjaminVanRyseghem 6/7/2012 12:28'!
registerEvents

	listHolder whenChangedDo: [ self refreshListItems ].
	filteringBlockHolder whenChangedDo: [ self refreshListItems ].
	selectionHolder index whenChangedDo: [ self changed: #getIndex ].
	multiSelection whenChangedDo: [ :b | self widget ifNotNil: [:w | w setMultipleSelection: b ] ].
	wrapBlockHolder whenChangedDo:[ self changed: #listElementAt: ].
	sortingBlockHolder
		whenChangedDo: [ 
			listHolder contents: (listHolder contents sorted: sortingBlockHolder contents).
			self changed: #listElementAt: ].
! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:22'!
resetFilteringBlock
	<api: #do>
	"Reset the filtering block with the default value which consists in showing everything"

	filteringBlockHolder contents: self defaultFilteringBlock! !

!ListComposableModel methodsFor: 'private-morphic' stamp: 'StephaneDucasse 5/17/2012 18:50'!
resetListSelection
	"Morphic API - The method should only be used internally for the communication between the model and the morphic widget" 
	"Reset the selection manager for multiple selection lists"
	
	multiSelectionHolder removeAll! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:22'!
resetSelection
	<api: #do>
	"Unselect every items"

	selectionHolder reset.
	multiSelectionHolder removeAll! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:22'!
resetSortingBlock
	<api: #do>
	"Reset the sortering block with the default value which consists in not sorting"

	sortingBlockHolder contents: self defaultSortingBlock! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:22'!
selectedIndex
	<api: #inspect>
	"Return the index of the selected item
	In the case of a multiple selection list, it returns the last selected item"

	^ self getIndex! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:22'!
selectedIndexes
	<api: #inspect>
	"Return the indexes of selected items on the case of a multiple selection list"
	
	^ multiSelectionHolder keys select: [:index | multiSelectionHolder at: index ]! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:23'!
selectedItem
	<api: #inspect>
	"Return the selected item.
	In the case of a multiple selection list, it returns the last selected item"
	
	^ self selectedItemHolder contents! !

!ListComposableModel methodsFor: 'private' stamp: 'StephaneDucasse 5/17/2012 18:04'!
selectedItemHolder

	^ selectionHolder selection! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:23'!
selectedItems
	<api: #inspect>
	"Return all the selected items in the case of a multiple selection list"

	^ multiSelectionHolder keys select: [:index | multiSelectionHolder at: index ] thenCollect: [:index | self listElementAt: index ]! !

!ListComposableModel methodsFor: 'private-morphic' stamp: 'StephaneDucasse 5/17/2012 19:18'!
setIndex: anIndex
	"Morphic API - The method should only be used internally for the communication between the model and the morphic widget" 
	"Set the index of the selected item when you click on an item"

	self allowToSelect ifFalse: [ ^ self ].
	self okToChange ifFalse: [ ^ self ].
	selectionHolder index contents: anIndex.
	selectionHolder selection contents: (self listElementAt: anIndex).! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 12/20/2012 16:21'!
setSelectedIndex: anIndex
	<api: #integer min: 0 max: #listSize getter: #selectedIndex registration: #whenSelectionIndexChanged:>
	"Set the index of the item you want to be selected"
	| idx selection |
	
	self allowToSelect ifFalse: [ ^ self ].
	self okToChange ifFalse: [ ^ self ].
	
	idx := anIndex min: self listSize.
	selection := self 
					listElementAt: idx 
					ifAbsent: [ idx := 0.
								nil ].

	selectionHolder index contents: idx.
	selectionHolder selection contents: selection.! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:24'!
setSelectedItem: anItem
	<api: #st getter: #selectedItem registration: #whenSelectedItemChanged:>
	"Set the item you want to be selected"
	| index |

	self okToChange ifFalse: [ ^ self ].
	index := self listItems identityIndexOf: anItem ifAbsent: [ ^ self ].
	selectionHolder index contents: index.
	selectionHolder selection contents: anItem! !

!ListComposableModel methodsFor: 'private-morphic' stamp: 'StephaneDucasse 5/17/2012 18:55'!
setSelectionStateFor: anIndex at: aBoolean
	"Morphic API - The method should only be used internally for the communication between the model and the morphic widget" 
	"Set the selection state of the item at index _anIndex_ in the case of multiple selection list"

	self allowToSelect ifFalse: [ ^ self ].
	^ multiSelectionHolder at: anIndex put: aBoolean! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:35'!
shortcuts
	<api: #inspect>
	"Return the block used to defined the shortcuts"
	
	^ shortcutsHolder contents! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:35'!
shortcuts: aBlock
	<api: #block getter: #shortcuts registration: #whenShortcutsChanged:>
	"Set the block used to defined the shortcuts"
	
	shortcutsHolder contents: aBlock! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:40'!
sortingBlock
	<api: #inspect>
	"Return the ordering of the items"
	
	^ sortingBlockHolder contents! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:40'!
sortingBlock: aBlock
	<api: #block getter: #sortingBlock registration: #whenSortingBlockChanged:>
	"To set the ordering of the items"
	
	sortingBlockHolder contents: aBlock! !

!ListComposableModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 02:24'!
updateList
	<api: #do>
	"Refresh the list"
	
	self changed: #listElementAt:.
	self resetSelection! !

!ListComposableModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/13/2012 02:25'!
whenAllowToSelectChanged: aBlock
	<api: #event>
	"Set a block to value when the value of allowToSelect has changed"
	
	allowToSelect whenChangedDo: aBlock! !

!ListComposableModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/13/2012 02:39'!
whenDisplayBlockChanged: aBlock
	<api: #event>
	"Set a block to value when the filtering block block has changed"
	
	wrapBlockHolder whenChangedDo: aBlock! !

!ListComposableModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/13/2012 02:38'!
whenFilteringBlockChanged: aBlock
	<api: #event>
	"Set a block to value when the filtering block block has changed"
	
	filteringBlockHolder whenChangedDo: aBlock! !

!ListComposableModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/13/2012 02:24'!
whenListChanged: aBlock
	<api: #event>
	"Specify a block to value after the contents of the list has changed"
	"Basically when you set a new list of items"

	listHolder whenChangedDo: aBlock! !

!ListComposableModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/13/2012 02:34'!
whenMenuChanged: aBlock
	<api: #event>
	"Set a block to value when the menu block has changed"
	
	menuHolder whenChangedDo: aBlock! !

!ListComposableModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/13/2012 02:43'!
whenMultiSelectionChanged: aBlock
	<api: #event>
	"Set a block to value when the multiSelection value has changed"
	
	multiSelection whenChangedDo: aBlock! !

!ListComposableModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/13/2012 02:24'!
whenSelectedItemChanged: aBlock
	<api: #event>
	"Set a block to value when the select item is changed"
	
	selectionHolder selection whenChangedDo: aBlock! !

!ListComposableModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/13/2012 02:24'!
whenSelectionChanged: aBlock
	<api: #event>
	"Set a block to value when the selection of the list has changed"

	selectionHolder whenChangedDo: aBlock.
	multiSelectionHolder whenChangedDo: aBlock.! !

!ListComposableModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/13/2012 02:24'!
whenSelectionIndexChanged: aBlock
	<api: #event>
	"Set a block to value when the selection index has changed"
	
	selectionHolder index whenChangedDo: aBlock! !

!ListComposableModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/13/2012 02:35'!
whenShortcutsChanged: aBlock
	<api: #event>
	"Set a block to value when the shortcuts block has changed"
	
	shortcutsHolder whenChangedDo: aBlock! !

!ListComposableModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/13/2012 02:40'!
whenSortingBlockChanged: aBlock
	<api: #event>
	"Set a block to value when the sorting block has changed"
	
	sortingBlockHolder whenChangedDo: aBlock! !

!ListComposableModel methodsFor: 'private-morphic' stamp: 'BenjaminVanRyseghem 7/14/2012 16:41'!
wrapItem: anObject index: index
	"Morphic API - The method should only be used internally for the communication between the model and the morphic widget" 
	"Return the item _anObject_ wrapped"

	^ wrapBlockHolder contents cull: anObject cull: index! !

ListComposableModel subclass: #MultiColumnListModel
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!MultiColumnListModel commentStamp: '<historical>' prior: 0!
A MultiColumnListModel is a spec model for Multi columns list!

!MultiColumnListModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 7/16/2012 17:37'!
defaultSpec
	<spec>
	^ {#MultiColumnListSpec.
			#wrapSelector:. 				#wrapItem:index:.
	    		#model:. 						#model.
			#getListSizeSelector:. 			#listSize.
			#getIndexSelector:. 			#getIndex.
			#setIndexSelector:. 			#setIndex:.
			#getSelectionListSelector:.		#getSelectionStateFor:.
			#setSelectionListSelector:.		#setSelectionStateFor:at:.
			#getListElementSelector:. 		#listElementAt:.
			#resetListSelector:.				#resetListSelection.
			#getMenuSelector:.			#menu:shifted:.
			#keystrokeSelector:.			#keyStroke:fromMorph:.
			#setMultipleSelection:.			{#model. #multiSelection}.
			#borderWidth:.						#(model borderWidth).
			#borderColor:.						#(model borderColor).
			#setBalloonText:.					{ #model . #help}.
			#hResizing:. 					#spaceFill.
			#vResizing:. 					#spaceFill}! !

!MultiColumnListModel methodsFor: 'remove me' stamp: 'StephaneDucasse 5/17/2012 18:03'!
getIndex

	^ selectionHolder index contents! !

!MultiColumnListModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/14/2012 16:41'!
listElementAt: anIndex

	^ self wrapItem: (self listItems at: anIndex) index: anIndex ! !

!MultiColumnListModel methodsFor: 'morphic' stamp: 'StephaneDucasse 5/17/2012 18:03'!
setIndex: anIndex

	self allowToSelect ifFalse: [ ^ self ].
	self okToChange ifFalse: [ ^ self ].
	selectionHolder index contents: anIndex.
	selectionHolder selection contents: (self listItems at: anIndex ifAbsent: [ nil ]).! !

!MultiColumnListModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 12/20/2012 16:22'!
setSelectedIndex: anIndex
	<api: #integer min: 0 max: #listSize getter: #selectedIndex registration: #whenSelectionIndexChanged:>
	"Set the index of the item you want to be selected"
	| idx selection |
	
	self allowToSelect ifFalse: [ ^ self ].
	self okToChange ifFalse: [ ^ self ].
	self listSize isZero ifTrue:  [ ^self ].
	idx := anIndex min: self listSize.
	selection := self listItems 
					at: idx 
					ifAbsent: [ idx := 0.
								nil ].
	
	selectionHolder index contents: idx.
	selectionHolder selection contents: selection.! !

AbstractBasicWidget subclass: #RadioButtonModel
	instanceVariableNames: 'actionWhenActivatedHolder actionWhenDesactivatedHolder stateHolder labelClickableHolder labelHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!RadioButtonModel commentStamp: '<historical>' prior: 0!
A RadioButtonModel is a spec model for RadioButton

!! The method privateSetState: is really private and should only be used by RadioButtonGroup !!

You can also have a look at RadioButtonGroupExample for a full example of how to use them with a group.!

!RadioButtonModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 7/13/2012 05:29'!
defaultSpec
	<spec>
	^ {#CheckboxSpec.
	    		#on:selected:changeSelected:. 	#model. #state. #state:.
			#label:.								{	#model. #label		}.
			#labelClickable:.					{	#model. #labelClickable}.
			#beRadioButton.
			#hResizing:. 						#shrinkWrap.
			#vResizing:. 						#shrinkWrap.
			#setBalloonText:.					{ #model . #help}.
			#borderWidth:.						#(model borderWidth).
			#borderColor:.						#(model borderColor).
			#eventHandler:.					{	#EventHandler. #on:send:to:. #keyStroke.	#keyStroke:fromMorph:. #model	}}! !

!RadioButtonModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 6/12/2012 12:36'!
title

	^ 'Radio Button'! !

!RadioButtonModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 6/12/2012 12:36'!
actionWhenActivatedHolder
	
	^ actionWhenActivatedHolder! !

!RadioButtonModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 6/12/2012 12:36'!
actionWhenDesactivatedHolder
	
	^ actionWhenDesactivatedHolder! !

!RadioButtonModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 16:56'!
activationAction: aBlock
	<api: #block getter: #activationAction registration: #whenActivationActionChanged:>
	"This method is used to set the action to perform when I am activated"
	
	actionWhenActivatedHolder contents: aBlock! !

!RadioButtonModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 16:50'!
click
	<api: #do>
	"Simulate a click on the radioButton.
	Used when the checkboc is a list item"

	self toggleState! !

!RadioButtonModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 17:05'!
desactivationAction: aBlock
	<api: #block getter: #desactivationAction registration: #whenDesactivationActionChanged:>
	"This method is used to set the action to perform when I am desactivated"
	
	actionWhenDesactivatedHolder contents: aBlock! !

!RadioButtonModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 6/12/2012 12:36'!
enabledHolder
	
	^ enabledHolder! !

!RadioButtonModel methodsFor: 'focus' stamp: 'BenjaminVanRyseghem 6/12/2012 12:36'!
eventStringForNextFocus
	"String describing the keystroke to perform to jump to the next widget"
	^ '<right>'! !

!RadioButtonModel methodsFor: 'focus' stamp: 'BenjaminVanRyseghem 6/12/2012 12:36'!
eventStringForPreviousFocus
	"String describing the keystroke to perform to jump to the previous widget"
	^ '<left>'! !

!RadioButtonModel methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/12/2012 12:36'!
initialize
	"initialization code for CheckBox"

	super initialize.
	
	actionWhenActivatedHolder := [] asValueHolder.
	actionWhenDesactivatedHolder := [] asValueHolder.
	enabledHolder := true asValueHolder.
	stateHolder := false asValueHolder.
	labelClickableHolder := true asValueHolder. 
	labelHolder := '' asValueHolder.
	stateHolder whenChangedDo: [:bool |
		bool
			ifTrue: actionWhenActivatedHolder contents
			ifFalse: actionWhenDesactivatedHolder contents.
		self changed: #state ].
	labelClickableHolder whenChangedDo: [:aBoolean |
		self widget ifNotNil: [:w | w labelClickable: aBoolean ]].
	labelHolder whenChangedDo: [:label |
		self widget ifNotNil: [:w | w label: label ]].
	! !

!RadioButtonModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/16/2012 17:06'!
label
	<api: #inspect>
	^ labelHolder contents! !

!RadioButtonModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 17:05'!
label: aString
	<api: #string getter: #label registration: #whenLabelChanged:>
	"Set the label of the radioButton"

	labelHolder contents: aString.! !

!RadioButtonModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/16/2012 17:06'!
labelClickable
	<api: #inspect>
	"Return true if the label can be clicked to select the radioButton"

	^ labelClickableHolder contents! !

!RadioButtonModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 17:04'!
labelClickable: aBoolean
	<api: #boolean getter: #labelClickable registration: #whenLabelClickableChanged:>
	"Set if the label can be clicked to select the radioButton"
	
	labelClickableHolder contents: aBoolean! !

!RadioButtonModel methodsFor: 'private' stamp: 'BenjaminVanRyseghem 7/11/2012 17:13'!
privateSetState: aBoolean
	"Should only be used by RadioButtonGroup"
	
	stateHolder contents: aBoolean.
	self changed: #state! !

!RadioButtonModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/16/2012 17:06'!
state
	<api: #inspect>
	"Return the current state of the radio button"

	^ stateHolder contents! !

!RadioButtonModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/16/2012 17:06'!
state: aBoolean
	<api: #boolean getter: #state>
	"Set if the checkbox is activated or not"

	stateHolder contents: aBoolean! !

!RadioButtonModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 6/12/2012 12:36'!
stateHolder
	
	^ stateHolder! !

!RadioButtonModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 6/12/2012 13:40'!
toggleState
	"Toogle the current state of the radioButton"

	self state: self state not! !

!RadioButtonModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/11/2012 17:13'!
whenActivatedDo: aBlock
	"This method is used to propagate the event that I have been activated" 
	
	stateHolder whenChangedDo: [:bool | bool ifTrue: aBlock ]! !

!RadioButtonModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 16:56'!
whenActivationActionChanged: aBlock
	<api: #event>
	"A block performed when the activation action changed"
	actionWhenActivatedHolder whenChangedDo:  aBlock ! !

!RadioButtonModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 16:57'!
whenChangedDo: aBlock
	<api: #event>
	"This method is used to propagate the event that I have changed"
	
	stateHolder whenChangedDo: aBlock! !

!RadioButtonModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/11/2012 17:13'!
whenDesactivatedDo: aBlock
	"This method is used to propagate the event that I have been desactivated"
	
	stateHolder whenChangedDo: [:bool | bool ifFalse: aBlock ]! !

!RadioButtonModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 16:59'!
whenDesactivationActionChanged: aBlock
	<api: #event>
	"A block performed when the desactivation action changed"
	actionWhenDesactivatedHolder whenChangedDo:  aBlock ! !

!RadioButtonModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 17:01'!
whenLabelChanged: aBlock
	"A block performed when the label changed"
	<api: #event>
	
	labelHolder whenChangedDo:  aBlock ! !

!RadioButtonModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 17:02'!
whenLabelClickableChanged: aBlock
	"A block performed when the label click is activated or descativated"
	<api: #event>
	
	labelClickableHolder whenChangedDo:  aBlock ! !

AbstractBasicWidget subclass: #SliderModel
	instanceVariableNames: 'max min quantum value isSetting isChanging absoluteValue labelHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!SliderModel commentStamp: '<historical>' prior: 0!
A SliderModel is a spec model for a slider!

!SliderModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 7/18/2012 16:45'!
defaultSpec
	<spec>
	
	^ SpecLayout slider
		send: #model: withArguments: #( model );
		send: #getValueSelector: withArguments: #(value);
		send: #setValueSelector: withArguments: #( value: );
		send: #value: withArguments: #(#(model absoluteValue));
		send: #getLabelSelector: withArguments: #( label );
		send: #max: withArguments: #(#(model max));
		send: #min: withArguments: #(#(model min));		
		send: #quantum: withArguments: #(#(model quantum));
		send: #borderWidth: withArguments: #(#(model borderWidth));
		send: #borderColor: withArguments: #(#(model borderColor));
		send:#setBalloonText: withArguments:	#(#(model help));
		send: #eventHandler: withArguments: #(#(EventHandler on:send:to: keyStroke keyStroke:fromMorph: model));
		send: #vSpaceFill;
		send: #hSpaceFill;
		yourself! !

!SliderModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/16/2012 17:14'!
absoluteValue
	<api: #inspect>
	"Return the position of the slider in a scale between 0 and 1 despite of the min and max value"
	^ absoluteValue contents! !

!SliderModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/16/2012 17:26'!
absoluteValue: aFloat
	<api: #float min: 0 max: 1 getter:#absoluteValue registration: #whenAbsoluteValueChanged:>
	"Set the position of the slider in a scale between 0 and 1 despite of the min and max value"

	absoluteValue contents: aFloat! !

!SliderModel methodsFor: 'as yet unclassified' stamp: 'BenjaminVanRyseghem 7/18/2012 18:59'!
color: aColor

	self widget ifNotNil: [:w | w == self ifFalse: [ super color: aColor ]]! !

!SliderModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/12/2012 19:17'!
defaultValue

	^ 50! !

!SliderModel methodsFor: 'initialize' stamp: 'BenjaminVanRyseghem 7/18/2012 16:47'!
initialize
	"Initialization code for SliderModel"

	super initialize.

	max := 10 asValueHolder.
	min := 0 asValueHolder.
	quantum := 1 asValueHolder.
	value := 5 asValueHolder.
	absoluteValue := 0.5 asValueHolder.
	labelHolder := '' asValueHolder.
	isChanging := false.
	
	max whenChangedDo: [:m | self widget ifNotNil: [:w | w max: m ]].
	min whenChangedDo: [:m | self widget ifNotNil: [:w | w min: m ]].
	quantum whenChangedDo: [:m | self widget ifNotNil: [:w | w quantum: m ]].
	
	value whenChangedDo: [:v |
		isChanging ifFalse: [
			isChanging := true.
			self absoluteValue: ((v-self min)/(self max-self min)).
			self widget ifNotNil: [:w | w == self ifFalse: [ w scaledValue: v ]].
			isChanging := false ]].
	
	absoluteValue whenChangedDo: [:v | 
		isChanging ifFalse: [
			isChanging := true.
			self value: (self min+(v*(self max-self min)) roundTo: self quantum).
			self widget ifNotNil: [:w | w == self ifFalse: [ w value: v ]].
			isChanging := false ]].
	
	labelHolder whenChangedDo: [	self widget ifNotNil: [:w | w updateLabel ]].! !

!SliderModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/18/2012 16:45'!
label
	<api: #inspect>
	"Return the label of the slider"
	
	^ labelHolder contents! !

!SliderModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/18/2012 16:46'!
label: aString
	<api: #string getter: #label registration: #whenLabelChanged:>
	"Set the label of the slider"
	
	labelHolder contents: aString! !

!SliderModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/16/2012 17:32'!
max
	<api: #inspect>
	"Return the maximun value"
	
	^ max contents! !

!SliderModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 17:34'!
max: anObject
	<api: #integer min: 1 max: 100 getter:#max registration: #whenMaxChanged:>
	"Set the maximun value"

	max contents: anObject! !

!SliderModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/16/2012 17:33'!
min
	<api: #inspect>
	"Return the minimum value"
	
	^ min contents! !

!SliderModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 17:34'!
min: anObject
	<api: #integer min: 0 max: 99 getter:#min registration: #whenMinChanged:>
	"Set the minimum value"
	
	min contents: anObject! !

!SliderModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/16/2012 17:33'!
quantum
	<api: #inspect>
	"Return the quantum betwen values"

	^ quantum contents! !

!SliderModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 17:33'!
quantum: aNumber
	<api: #integer min: #min max: #max getter:#quantum registration: #whenQuantumChanged:>
	"Set the quantum betwen values"

	quantum contents: aNumber! !

!SliderModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 17:57'!
reset
	<api:#do>
	"Reset the cursor to the minimum value"
	
	self value: self min! !

!SliderModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/16/2012 17:33'!
value
	<api: #inspect>
	"Return the current value in a range between min and max"

	^ value contents! !

!SliderModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 17:30'!
value: aNumber
	<api: #integer min: #min max: #max getter:#value registration: #whenValueChanged:>
	"Set the value in a range between min and max"

	value contents: aNumber.
	self changed: #value.! !

!SliderModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 17:35'!
whenAbsoluteValueChangedDo: aBlock
	<api: #event>
	"Block performed when the absolute value changed"

	absoluteValue whenChangedDo: aBlock! !

!SliderModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/18/2012 16:46'!
whenLabelChangedDo: aBlock
	<api: #event>
	"Block performed when the label changed"

	labelHolder whenChangedDo: aBlock! !

!SliderModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 17:35'!
whenMaxChangedDo: aBlock 
	<api: #event>
	"Block performed when the maximum value changed"

	max whenChangedDo: aBlock! !

!SliderModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 17:35'!
whenMinChangedDo: aBlock 
	<api: #event>
	"Block performed when the minimum value changed"

	min whenChangedDo: aBlock! !

!SliderModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 17:35'!
whenQuantumChangedDo: aBlock 
	<api: #event>
	"Block performed when the quantum value changed"

	quantum whenChangedDo: aBlock! !

!SliderModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 17:35'!
whenValueChangedDo: aBlock 
	<api: #event>
	"Block performed when the value changed"

	value whenChangedDo: aBlock! !

AbstractBasicWidget subclass: #TextModel
	instanceVariableNames: 'aboutToStyleHolder behaviorHolder textHolder actionToPerformHolder readSelectionHolder actionPerformedHolder isCodeCompletionAllowedHolder doItContext doItReceiver'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!TextModel commentStamp: '<historical>' prior: 0!
A TextComposableModel is an applicative model which handle a basic text.

| t |
t:= TextComposableModel new.
t openWithSpec.
t textHolder contents: 'This is a test'.

self example!

TextModel subclass: #TextInputFieldModel
	instanceVariableNames: 'ghostText alwaysAccept acceptOnCR autoAccept entryCompletion isEncrypted'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!TextInputFieldModel commentStamp: '<historical>' prior: 0!
A TextInputFieldModel is a spec model for TextInputField.!

!TextInputFieldModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 8/22/2012 15:09'!
defaultSpec
	<spec>
	^ {#TextField.
			#convertTo:. 							String.
			#on:text:accept:readSelection:menu:. 	#model. #getText. #accept:. nil. nil.
			#entryCompletion:. 				{	#model. 	#entryCompletion	}.
			#autoAccept:.						{	#model. 	#autoAccept			}.	
			#ghostText:.						{	#model. 	#ghostText			}.
			#enabled:.							{	#model. 	#enabled				}.
			#encrypted:.						{ 	#model.	#encrypted			}.
			#borderWidth:.						#(model borderWidth).
			#borderColor:.						#(model borderColor).
			#setBalloonText:.					{ #model . #help}.
			#hResizing:. 							#spaceFill.
			#vResizing:. 							#spaceFill.
			#acceptOnCR:. 					{	#model. 	#acceptOnCR			}.
			#hideScrollBarsIndefinitely.
			#eventHandler:.					{	#EventHandler. #on:send:to:. #keyStroke.	#keyStroke:fromMorph:. #model	}}! !

!TextInputFieldModel class methodsFor: 'example' stamp: ''!
example
	| instance |
	
	instance := self new.
	instance openWithSpec.
	instance ghostText: 'This is a ghost'.
	instance text: 'This is a text'! !

!TextInputFieldModel class methodsFor: 'specs' stamp: ''!
title

	^ 'Text'! !

!TextInputFieldModel methodsFor: 'morphic' stamp: 'StephaneDucasse 5/17/2012 17:45'!
accept: aText

	textHolder contents: aText.
	actionToPerformHolder contents cull: aText.
	self changed: #clearUserEdits.! !

!TextInputFieldModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 8/22/2012 15:09'!
acceptOnCR
	<api: #inspect>
	"Return true if the text is accepted when the Enter key is stroked"

	^ acceptOnCR contents! !

!TextInputFieldModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/19/2012 21:57'!
acceptOnCR: aBoolean
	<api: #boolean getter: #acceptOnCR registration: #whenAcceptOnCRChanged:>
	"Set if the text is accepted when the Enter key is stroked or not"

	acceptOnCR contents: aBoolean! !

!TextInputFieldModel methodsFor: 'accessing' stamp: ''!
actionToPerformHolder

	^ actionToPerformHolder! !

!TextInputFieldModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/19/2012 21:59'!
autoAccept 
	<api: #inspect>
	"Return true if the text is accepted after ech keystroke"

	^ autoAccept contents! !

!TextInputFieldModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/19/2012 21:59'!
autoAccept: aBoolean
	<api: #boolean getter: #autoAccept registration: #whenAutoAcceptChanged:>
	"Set if the text is accepted after each keystroke or not"

	autoAccept contents: aBoolean.! !

!TextInputFieldModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/19/2012 22:01'!
beDecrypted
	<api: #do>
	"Decrypt the entered text"
	
	self encrypted: false! !

!TextInputFieldModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/19/2012 22:02'!
beEncrypted
	<api: #do>
	"Encrypt the entered text"
	
	self encrypted: true! !

!TextInputFieldModel methodsFor: 'private' stamp: 'EstebanLorenzano 8/17/2012 16:41'!
defaultEntryCompletion

	| applicants |
	applicants := (Array
				streamContents: [:strm | Symbol
						allSymbolTablesDo: [:each | (each notEmpty
									and: [each first canBeGlobalVarInitial
											and: [Smalltalk globals includesKey: each]])
								ifTrue: [strm nextPut: each]]]) sort.

	^ EntryCompletion new
				dataSourceBlock: [:currText | applicants];
				filterBlock: [:currApplicant :currText | currText size > 3
						and: [currApplicant asUppercase includesSubstring: currText asString asUppercase]].! !

!TextInputFieldModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 7/4/2012 14:52'!
encrypted

	^ isEncrypted contents! !

!TextInputFieldModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 7/4/2012 14:53'!
encrypted: aBoolean

	isEncrypted contents: aBoolean ! !

!TextInputFieldModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/19/2012 22:03'!
entryCompletion
	<api: #st getter: #entryCompletion registration: #whenEntryCompletionChanged:>
	"Return an entry completion used to suggest text while typing"

	^ entryCompletion contents! !

!TextInputFieldModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/19/2012 22:03'!
entryCompletion: anEntryCompletion
	<api: #st getter: #entryCompletion registration: #whenEntryCompletionChanged:>
	"Set an entry completion used to suggest text while typing"

	entryCompletion contents: anEntryCompletion.! !

!TextInputFieldModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/19/2012 22:04'!
ghostText
	<api: #inspect>
	"Return the ghost text of the text zone"

	^ ghostText contents! !

!TextInputFieldModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/19/2012 22:05'!
ghostText: aText
	<api: #string getter: #ghostText registration: #whenGhostTextChanged:>
	"Set the ghost text of the text zone"

	ghostText contents: aText translated.! !

!TextInputFieldModel methodsFor: 'initialization' stamp: 'PavelKrivanek 8/22/2012 17:10'!
initialize
	"Initialization code for TextComposableModel"

	super initialize.
	
	ghostText := '' asValueHolder.
	acceptOnCR := true asValueHolder.
	autoAccept := false asValueHolder.
	entryCompletion := self defaultEntryCompletion asValueHolder.
	isEncrypted := false asValueHolder.
	
	ghostText  whenChangedDo: [:txt | self widget ifNotNil: [:w | w ghostText: txt ]].
	acceptOnCR whenChangedDo: [:bool | self widget ifNotNil: [:w | w acceptOnCR: bool ]].
	autoAccept whenChangedDo: [:bool | self widget ifNotNil: [:w | w autoAccept: bool ]].
	isEncrypted whenChangedDo: [:bool | self widget ifNotNil: [:w | w encrypted: bool ]].! !

!TextInputFieldModel methodsFor: 'NOCompletion' stamp: 'BenjaminVanRyseghem 6/25/2012 23:46'!
isCodeCompletionAllowed
	
	^ false! !

!TextInputFieldModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/19/2012 22:06'!
removeEntryCompletion
	< api:#do>
	"Remove the entry completion"
	
	self entryCompletion: nil! !

!TextInputFieldModel methodsFor: 'accessing' stamp: ''!
textHolder

	^ textHolder! !

!TextInputFieldModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/19/2012 21:58'!
whenAcceptOnCRChanged: aBlock
	<api: #event>
	"aBlock will be performed when acceptOnCR changed"
	
	acceptOnCR whenChangedDo: aBlock! !

!TextInputFieldModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/19/2012 22:00'!
whenAutoAcceptChanged: aBlock
	<api: #event>
	"aBlock will be performed when autoAccept changed"
	
	autoAccept whenChangedDo: aBlock! !

!TextInputFieldModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/19/2012 22:04'!
whenEntryCompletionChanged: aBlock
	<api: #event>
	"aBlock will be performed when entryCompletion changed"
	
	entryCompletion whenChangedDo: aBlock! !

!TextInputFieldModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/19/2012 22:05'!
whenGhostTextChanged: aBlock
	<api: #event>
	"aBlock will be performed when ghostText changed"
	
	ghostText whenChangedDo: aBlock! !

!TextModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 7/13/2012 05:30'!
defaultSpec
	<spec>
	
	^ SpecLayout text
			send: #on:text:accept:readSelection:menu: withArguments: #(model getText accept:notifying: readSelection codePaneMenu:shifted:);
			send: #classOrMetaClass: withArguments: {#(model behavior)};
			send: #enabled: withArguments: {#(model enabled)};
			send: #borderWidth: withArguments: #(#(model borderWidth));
			send: #borderColor: withArguments: #(#(model borderColor));
			send:#setBalloonText: withArguments:	{ #(model help)};
			send: #eventHandler: withArguments: {#(EventHandler on:send:to: keyStroke keyStroke:fromMorph: model)};
			send: #vSpaceFill;
			send: #hSpaceFill;
			yourself! !

!TextModel class methodsFor: 'example' stamp: ''!
example
	| instance |
	
	instance := self new.
	instance openWithSpec.
	instance text: 'This is a test'.! !

!TextModel class methodsFor: 'specs' stamp: ''!
title

	^ 'Text'! !

!TextModel methodsFor: 'protocol-shout' stamp: 'BenjaminVanRyseghem 7/16/2012 22:12'!
aboutToStyle: aBoolean	
	<api: #boolean getter: #isAboutToStyle registration: #whenAboutToStyleChanged:>
	"Set if the text zone must be styled"
	
	aboutToStyleHolder contents: [ aBoolean ]! !

!TextModel methodsFor: 'protocol-shout' stamp: 'BenjaminVanRyseghem 7/16/2012 20:11'!
aboutToStyleBlock
	<api: #inspect>
	"Return the block used to know if the text must be styled"	

	^ aboutToStyleHolder contents! !

!TextModel methodsFor: 'protocol-shout' stamp: 'BenjaminVanRyseghem 7/16/2012 22:21'!
aboutToStyleBlock: aBlockReturningABoolean
	<api: #block getter: #aboutToStyleBlock registration: #whenAboutToStyleBlockChanged:>
	"Set the block used to know if the text must be styled.
	The block must return a boolean"

	aboutToStyleHolder contents: aBlockReturningABoolean! !

!TextModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 19:37'!
accept
	<api: #do>
	"Accep the current pendingtext"
	
	self widget ifNotNil: [:w | w accept ]! !

!TextModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 6/12/2012 13:51'!
accept: aText notifying: aNotifyier

	textHolder contents: aText.
	actionToPerformHolder contents cull: aText cull: aNotifyier.
	
	"Dummy changed to raise an announcement"
	actionPerformedHolder contents: nil.
	self changed: #clearUserEdits.! !

!TextModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 22:13'!
acceptBlock
	<api: #inspect>
	"Return the block to perform when the text is accepted.
	The block must have one argument, which will be the accepted text"

	^ actionToPerformHolder contents! !

!TextModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 22:13'!
acceptBlock: aBlockWithOneArgument
	<api: #block getter: #acceptBlock registration: #whenAcceptBlockChanged:>
	"Set the block to perform when the text is accepted.
	The block must have one argument, which will be the accepted text"

	actionToPerformHolder contents: aBlockWithOneArgument! !

!TextModel methodsFor: 'protocol-shout' stamp: 'BenjaminVanRyseghem 7/16/2012 20:12'!
behavior
	<api: #inspect>
	"Return the class corresponding to the method class of the source code you are editing"
	
	^ behaviorHolder contents! !

!TextModel methodsFor: 'protocol-shout' stamp: 'BenjaminVanRyseghem 7/16/2012 20:13'!
behavior: aClass
	< api: #st getter: #behavior registration: #whenBehaviorChanged:>
	"Set the class corresponding to the method class of the source code you are editing"

	behaviorHolder contents: aClass! !

!TextModel methodsFor: 'morphic' stamp: ''!
clearUserEditFlag
	
	self text: self getText.
	self changed: #clearUserEdits! !

!TextModel methodsFor: 'morphic' stamp: ''!
codePaneMenu: aMenu shifted: shifted 
	"Note that unless we override perform:orSendTo:, 
	PluggableTextController will respond to all menu items in a 
	text pane"
	
	^ shifted
			ifTrue: [SmalltalkEditor shiftedYellowButtonMenu]
			ifFalse: [SmalltalkEditor yellowButtonMenu].! !

!TextModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 9/28/2012 11:02'!
doItContext

	^ doItContext contents! !

!TextModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 9/28/2012 11:03'!
doItContext: anObject

	doItContext contents: anObject! !

!TextModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 9/28/2012 11:03'!
doItReceiver

	^ doItReceiver contents! !

!TextModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 9/28/2012 11:03'!
doItReceiver: anObject

	doItReceiver contents: anObject! !

!TextModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 17:47'!
eventStringForNextFocus
	"String describing the keystroke to perform to jump to the next widget"
	^ '<Ctrl-Cmd-right>'! !

!TextModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 17:47'!
eventStringForPreviousFocus
	"String describing the keystroke to perform to jump to the previous widget"

	^ '<Ctrl-Cmd-left>'! !

!TextModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 19:38'!
getSelection
	<api:#inspect>
	"Get the text selection"

	self widget
		ifNil: [ self readSelection ]
		ifNotNil: [:w | w selectionInterval ]! !

!TextModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/16/2012 20:10'!
getText
	<api: #inspect>	
	^ textHolder contents! !

!TextModel methodsFor: 'NOCompletion' stamp: 'BenjaminVanRyseghem 6/22/2012 14:17'!
guessTypeForName: aString

	^nil! !

!TextModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 19:38'!
hasEditingConflicts
	<api:#inspect>
	"Return if the text zone has editing conflicts"

	^ self widget 
		ifNil: [ false ]
		ifNotNil: [:w | w hasEditingConflicts ]! !

!TextModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 19:39'!
hasEditingConflicts: aBoolean
	<api:#boolean getter: #hasEditingConflicts>
	"Set if the text zone has editing conflicts"

	^ self widget 
		ifNil: [ false ]
		ifNotNil: [:w | w hasEditingConflicts: aBoolean ]! !

!TextModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 20:03'!
hasUnacceptedEdits
	<api:#inspect>
	"Return if the text zone has unaccepted edits (orange corner)"

	^ self widget 
		ifNil: [ false ]
		ifNotNil: [:w | w hasUnacceptedEdits ]! !

!TextModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 20:03'!
hasUnacceptedEdits: aBoolean
	<api:#boolean getter: #hasUnacceptedEdits>
	"Return if the text zone has unaccepted edits (orange corner)"

	^ self widget 
		ifNil: [ false ]
		ifNotNil: [:w | w hasUnacceptedEdits: aBoolean ]! !

!TextModel methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 9/28/2012 11:02'!
initialize
	"Initialization code for TextComposableModel"

	super initialize.
	
	textHolder := '' asValueHolder.
	behaviorHolder := nil asValueHolder.
	aboutToStyleHolder := [ false ] asValueHolder.
	actionToPerformHolder := [ :text |  ] asValueHolder.
	readSelectionHolder := [ 1 to: 0 ] asValueHolder.
	enabledHolder := true asValueHolder.
	actionPerformedHolder := nil asValueHolder.
	isCodeCompletionAllowedHolder := true asValueHolder.
	doItContext := nil asValueHolder.
	doItReceiver := nil asValueHolder.
	
	self registerEvents.
	self registerEventsForShout! !

!TextModel methodsFor: 'protocol-shout' stamp: 'BenjaminVanRyseghem 7/16/2012 20:13'!
isAboutToStyle
	< api: #inspect>
	"Return if the text zone is shouted or not"

	^ self aboutToStyleBlock value! !

!TextModel methodsFor: 'NOCompletion' stamp: 'BenjaminVanRyseghem 7/16/2012 20:04'!
isCodeCompletionAllowed
	<api:#inspect>
	"Return if code completion is allowed"
	
	^ isCodeCompletionAllowedHolder contents! !

!TextModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 20:05'!
isCodeCompletionAllowed: aBoolean
	<api: #boolean getter: #isCodeCompletionAllowed registration: #whenCodeCompletionAllowedChanged:>
	"Set if code completion is allowed"
	
	isCodeCompletionAllowedHolder contents: aBoolean! !

!TextModel methodsFor: 'private-shout' stamp: ''!
okToStyle

	^ true! !

!TextModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 20:05'!
pendingText
	<api: #inspect>
	"Return the current pending text"

	self widget
		ifNil: [ ^ self getText ]
		ifNotNil: [:w |  ^ w text ]! !

!TextModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 20:07'!
pendingText: aText
	<api: #string getter: #pendingText>
	"Set the pending text. Do no accept it"

	self widget
		ifNil: [ self text: aText ]
		ifNotNil: [:w |
			w setText: aText.
			w hasUnacceptedEdits: true ]! !

!TextModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/16/2012 20:10'!
readSelection
	<api: #inspect>
	^ readSelectionHolder contents cull: (self widget ifNil: [ nil ] ifNotNil: [:w | w text ]) cull: self! !

!TextModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 20:07'!
readSelectionBlock
	<api: #inspect>
	"Return the block used to calculate the text selection"

	^ readSelectionHolder contents! !

!TextModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 20:08'!
readSelectionBlock: aBlock
	<api: #block getter: #readSelectionBlock registration: #whenReadSelectionIsChanged:>
	"Set the block used to calculate the text selection"

	^ readSelectionHolder contents: aBlock! !

!TextModel methodsFor: 'NOCompletion' stamp: 'BenjaminVanRyseghem 6/22/2012 14:18'!
receiverClass

	^ self behavior! !

!TextModel methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/7/2012 12:29'!
registerEvents

	textHolder whenChangedDo: [ self changed: #getText ].
	readSelectionHolder whenChangedDo: [ self changed: #readSelection ].
	enabledHolder whenChangedDo: [ :bool | self widget ifNotNil: [:w | w enabled: bool ] ]! !

!TextModel methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/7/2012 12:29'!
registerEventsForShout
	aboutToStyleHolder whenChangedDo: [ self changed: #getText ].
	behaviorHolder
		whenChangedDo: [ :class | 
			self widget ifNotNil: [ :w | w classOrMetaClass: class ].
			self changed: #getText ]! !

!TextModel methodsFor: 'morphic' stamp: 'StephaneDucasse 5/17/2012 17:45'!
selectedBehavior

	^ behaviorHolder contents! !

!TextModel methodsFor: 'NOCompletion' stamp: 'BenjaminVanRyseghem 6/22/2012 14:19'!
selectedClassOrMetaClass

	^ self behavior! !

!TextModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 20:08'!
setSelection: anInterval
	<api: #st getter: #getSelection>
	"Set the text selection without changing the readSelectionBlock"

	self widget
		ifNotNil: [:w | w selectionInterval: anInterval ]! !

!TextModel methodsFor: 'morphic' stamp: 'StephaneDucasse 5/17/2012 17:44'!
shoutAboutToStyle: aMorph
	
	^ self isAboutToStyle! !

!TextModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/16/2012 20:09'!
text: aText
	<api: #string getter: #getText registration: #whenTextChanged:>
	"Set the text of the text zone"
	
	textHolder contents: aText! !

!TextModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 22:21'!
whenAboutToStyleBlockChanged: aBlock
	<api: #event>
	"Set a block to perform when the about to style block changed"
	
	aboutToStyleHolder whenChangedDo: aBlock! !

!TextModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 22:24'!
whenAboutToStyleChanged: aBlock
	<api: #event>
	"Set a block to perform when the about to style block changed"
	
	self whenAboutToStyleBlockChanged: [:block :old :announcement :announcer | 
		aBlock cull: (block value) cull: old cull: announcement cull: announcer ]! !

!TextModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 22:14'!
whenAcceptBlockChanged: aBlock
	<api: #event>
	"Set a block to perform when the accept block changed"
	
	actionToPerformHolder whenChangedDo: aBlock! !

!TextModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 20:13'!
whenBehaviorChanged: aBlock
	<api: #event>
	"Set a block to perform when the behavior class changed"
	
	behaviorHolder whenChangedDo: aBlock! !

!TextModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 22:11'!
whenCodeCompletionAllowedChanged: aBlock
	<api: #event>
	"Set a block to perform when the code completion value changed"
	
	actionPerformedHolder whenChangedDo: aBlock! !

!TextModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 20:11'!
whenReadSelectionIsChanged: aBlock
	<api: #event>
	"Set a block to perform when the read selection block changed"
	
	readSelectionHolder whenChangedDo: aBlock! !

!TextModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 22:11'!
whenTextChanged: aBlock
	<api: #event>
	"Set a block to perform when the text changed"
	
	textHolder whenChangedDo: aBlock! !

!TextModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/16/2012 22:11'!
whenTextIsAccepted: aBlock
	<api: #event>
	"Set a block to perform when the text is accepted"
	
	actionPerformedHolder whenChangedDo: aBlock! !

AbstractBasicWidget subclass: #TreeModel
	instanceVariableNames: 'childrenBlockHolder rootsHolder selectionHolder menuBlockHolder wrapBlockHolder'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!TreeModel commentStamp: '<historical>' prior: 0!
A TreeModel is a spec model of a Tree ;)!

!TreeModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 7/13/2012 05:29'!
defaultSpec
	<spec>
	
	^ {#TreeSpec.
	    		#model:.							#model.
			#getLabelSelector:.				#wrapItem:.
			#getRootsSelector:.				#getRoots.
			#getChildrenSelector:.				#getChildrenFor:.
			#setSelectedSelector:.			#setSelectedItem:.
			#color:.							Color white.
			#getMenuSelector:.				#getMenu:.
			#hResizing:. 						#spaceFill.
			#setBalloonText:.					{ #model . #help}.
			#borderColor:.						#(model borderColor).
			#borderWidth:.						#(model borderWidth).
			#vResizing:. 						#spaceFill.
			#eventHandler:.					{	#EventHandler. #on:send:to:. #keyStroke.	#keyStroke:fromMorph:. #model	}}! !

!TreeModel class methodsFor: 'example' stamp: 'bvr 6/2/2012 12:46'!
example
	| m |

	m := TreeModel new.
	m openWithSpec.
	m roots: #(a b c d).
	m childrenBlock: [:it | it = #c 
							ifTrue: [ {1. 2. 3} ]
							ifFalse: [ {} ]].
	m displayBlock: [:item | item isSymbol
								ifTrue: [ item, ' symbol' ]
								ifFalse: [ item printString ]].! !

!TreeModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 4/13/2012 14:38'!
title

	^ 'Tree'! !

!TreeModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/19/2012 22:10'!
childrenBlock
	< api: #inspect>
	"Return the block used to retrieve the children of a node.
	The optional block arguments are:
		- the node
		- the tree"

	^ childrenBlockHolder contents! !

!TreeModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/19/2012 22:11'!
childrenBlock: aBlock
	< api: #block getter: #childrenBlock registration: #whenChildrenBlockChanged:>
	"Set the block used to retrieve the children of a node.
	The optional block arguments are:
		- the node
		- the tree"

	childrenBlockHolder contents: aBlock! !

!TreeModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/19/2012 22:10'!
displayBlock
	< api: #inspect>
	"Return the block used to generate the display of the items.
	The optioanl block arguments are:
		- the item
		- the tree"

	^ wrapBlockHolder contents! !

!TreeModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/19/2012 22:12'!
displayBlock: aBlock
	< api: #block getter: #displayBlock registration: #whenDisplayBlockChanged:>
	"Set the block used to generate the display of the items.
	The optioanl block arguments are:
		- the item
		- the tree"

	wrapBlockHolder contents: aBlock! !

!TreeModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 18:17'!
eventStringForNextFocus
	"String describing the keystroke to perform to jump to the next widget"
	^ '<right>'! !

!TreeModel methodsFor: 'focus' stamp: 'StephaneDucasse 5/17/2012 18:18'!
eventStringForPreviousFocus
	"String describing the keystroke to perform to jump to the previous widget"
	^ '<left>'! !

!TreeModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 4/13/2012 14:24'!
getChildrenFor: anItem

		^ childrenBlockHolder contents cull: anItem cull: self! !

!TreeModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 4/13/2012 14:29'!
getMenu: aMenu

	menuBlockHolder contents cull: aMenu cull: self.
	^ aMenu! !

!TreeModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 6/6/2012 20:45'!
getRoots
							
	^ rootsHolder contents! !

!TreeModel methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 4/13/2012 14:36'!
initialize
	"Initialization code for TreeModel"

	super initialize.

	childrenBlockHolder := [ {} ] asValueHolder.
	rootsHolder := {} asValueHolder.
	selectionHolder := nil asValueHolder.
	menuBlockHolder := [] asValueHolder.
	wrapBlockHolder := [:item | item printString ] asValueHolder.

	self registerEvents! !

!TreeModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/19/2012 22:13'!
menu
	<api: #inspect>
	"Return the block used to generate the tree menu"

	^ menuBlockHolder contents! !

!TreeModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/19/2012 22:14'!
menu: aBlock
	< api: #block getter: #menu registration: #whenMenuChanged:>
	"Set the block used to generate the tree menu"

	menuBlockHolder contents: aBlock! !

!TreeModel methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/13/2012 16:31'!
registerEvents

	rootsHolder whenChangedDo: [ self changed: #getRoots ].
	childrenBlockHolder whenChangedDo: [ self changed: #getRoots ].
	wrapBlockHolder whenChangedDo: [ self changed: #getRoots ].! !

!TreeModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/19/2012 22:15'!
roots
	< api: #inspect>
	"Return the tree roots"

	^ rootsHolder contents! !

!TreeModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/19/2012 22:15'!
roots: aCollection
	< api: #st getter: #roots registration: #whenRootsChanged:>
	"Set the tree roots"

	rootsHolder contents: aCollection! !

!TreeModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/19/2012 22:16'!
selectedItem
	< api: #inspect>
	"Return the selected item"
	"In the case of a multiple selection list, it returns the last selected item"
	
	^ selectionHolder contents! !

!TreeModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/19/2012 22:17'!
selectedItem: anItem
	< api: #st getter: #selectedItem registration: #whenSelectedItemChanged:>
	"Set the selected item"
	"In the case of a multiple selection list, it returns the last selected item"
	
	selectionHolder contents: anItem! !

!TreeModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 4/13/2012 14:30'!
setSelectedItem: anItem

	selectionHolder contents: anItem.! !

!TreeModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/19/2012 22:18'!
updateTree
	< api: #do>
	"Force the tree to refresh"

	self changed: #getRoots! !

!TreeModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/19/2012 22:11'!
whenChildrenBlockChanged: aBlock
	"Set a block to perform when a new childrenBlock is set"

	childrenBlockHolder whenChangedDo: aBlock! !

!TreeModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/19/2012 22:12'!
whenDisplayBlockChanged: aBlock
	"Set a block to perform when a new displayBlock is set"

	wrapBlockHolder whenChangedDo: aBlock! !

!TreeModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 7/19/2012 22:14'!
whenMenuChanged: aBlock
	"Set a block to perform when a new menuBlock is set"

	menuBlockHolder whenChangedDo: aBlock! !

!TreeModel methodsFor: 'protocol-events' stamp: 'BenjaminVanRyseghem 6/12/2012 14:03'!
whenSelectedItemChanged: aBlock
	"Set a block to perform when a new item is selected"

	selectionHolder whenChangedDo: aBlock! !

!TreeModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 4/13/2012 14:35'!
wrapItem: anItem

	^ wrapBlockHolder contents cull: anItem cull: self! !

AbstractBasicWidget subclass: #WindowModel
	instanceVariableNames: 'model initialExtent title'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!WindowModel commentStamp: '<historical>' prior: 0!
A WindowModel is a spec model for a Window!

WindowModel subclass: #DialogWindowModel
	instanceVariableNames: 'contentMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets'!

!DialogWindowModel commentStamp: '<historical>' prior: 0!
A DialogWindowModel is a model used to describe a DialogWindow!

!DialogWindowModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 7/10/2012 16:22'!
defaultSpec
	<spec>
	
	^ #( DialogWindowSpec 
			specWidget: #(model contentMorph)
			initialize
			model: model )! !

!DialogWindowModel methodsFor: 'private' stamp: 'BenjaminVanRyseghem 10/18/2012 15:21'!
buildWithSpec: aSpec
	"Build the widget using the spec name provided as argument"
	| widget |
	
	(self spec notNil and: [ self needRebuild not ])
		ifTrue: [ widget := self spec instance ]
		ifFalse: [ 
			contentMorph := self model ifNil: [ nil ] ifNotNil: [:m | m buildWithSpec: aSpec ].
			widget := SpecInterpreter buildWidgetFor: self withSpec: self defaultSpecSelector.
			contentMorph := nil ].

	self extent ifNotNil: [:ex | 
		(widget respondsTo: #extent:)
			ifTrue: [ widget extent: ex ]].
		
	^ widget! !

!DialogWindowModel methodsFor: 'private' stamp: 'BenjaminVanRyseghem 10/18/2012 15:21'!
buildWithSpecLayout: aSpec
	"Build the widget using the spec name provided as argument"
	| widget |
	
	(self spec notNil and: [ self needRebuild not ])
		ifTrue: [ widget := self spec instance ]
		ifFalse: [ 
			contentMorph := self model buildWithSpecLayout: aSpec.
			widget := SpecInterpreter buildWidgetFor: self withSpec: self defaultSpecSelector.
			contentMorph := nil ].

	self extent ifNotNil: [:ex | 
		(widget respondsTo: #extent:)
			ifTrue: [ widget extent: ex ]].
		
	^ widget! !

!DialogWindowModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/10/2012 16:25'!
cancelAction: aBlock

	^ self widget
		ifNotNil: [:w | w cancelAction: aBlock ]! !

!DialogWindowModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/10/2012 16:25'!
cancelled

	^ self widget
		ifNil: [ false ]
		ifNotNil: [:w | w cancelled ]! !

!DialogWindowModel methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 7/10/2012 16:14'!
contentMorph

	^ contentMorph! !

!DialogWindowModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/10/2012 16:25'!
okAction: aBlock

	^ self widget
		ifNotNil: [:w | w okAction: aBlock ]! !

!DialogWindowModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/10/2012 16:25'!
toolbar

	^ self widget
		ifNotNil: [:w | w toolbar ]! !

!WindowModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 7/10/2012 15:09'!
defaultSpec
	<spec>
	
	^ #( WindowSpec model: model )! !

!WindowModel methodsFor: 'private' stamp: 'BenjaminVanRyseghem 7/31/2012 14:42'!
addModelIn: widget withSpec: aSpec

	self model ifNil: [ ^ self ].
	
	widget
		addMorph: (self model buildWithSpec: aSpec)
		frame: (0@0 corner: 1@1)! !

!WindowModel methodsFor: 'private' stamp: 'BenjaminVanRyseghem 7/10/2012 15:43'!
addModelIn: widget withSpecLayout: aSpec

	self model ifNil: [ ^ self ].
	widget
		addMorph: (self model buildWithSpecLayout: aSpec)
		frame: (0@0 corner: 1@1)! !

!WindowModel methodsFor: 'private' stamp: 'BenjaminVanRyseghem 12/12/2012 15:09'!
buildWithSpec: aSpec
	"Build the widget using the spec name provided as argument"
	| widget |
	
	(self spec notNil and: [ self needRebuild not ])
		ifTrue: [ widget := self spec instance ]
		ifFalse: [ 
			widget := SpecInterpreter private_buildWidgetFor: self withSpec: self defaultSpecSelector.
			self addModelIn: widget withSpec: aSpec ].

	self extent ifNotNil: [:ex | 
		(widget respondsTo: #extent:)
			ifTrue: [ widget extent: ex ]].
		
	^ widget! !

!WindowModel methodsFor: 'private' stamp: 'BenjaminVanRyseghem 12/12/2012 15:09'!
buildWithSpecLayout: aSpec
	"Build the widget using the spec name provided as argument"
	| widget |
	
	(self spec notNil and: [ self needRebuild not ])
		ifTrue: [ widget := self spec instance ]
		ifFalse: [
			widget := SpecInterpreter private_buildWidgetFor: self withSpec: self defaultSpecSelector.
			self addModelIn: widget withSpecLayout: aSpec ].

	self extent ifNotNil: [:ex | 
		(widget respondsTo: #extent:)
			ifTrue: [ widget extent: ex ]].
		
	^ widget! !

!WindowModel methodsFor: 'protocol' stamp: 'CamilloBruni 9/21/2012 14:03'!
close
	self widget ifNotNil: [ :w| w close ].! !

!WindowModel methodsFor: 'private' stamp: 'BenjaminVanRyseghem 7/10/2012 15:15'!
defaultInitialExtent
	
	^ 400@300! !

!WindowModel methodsFor: 'focus' stamp: 'BenjaminVanRyseghem 8/2/2012 16:55'!
giveFocusToNextFrom: aMorph 

	^ self model giveFocusToNextFrom: aMorph! !

!WindowModel methodsFor: 'focus' stamp: 'BenjaminVanRyseghem 7/10/2012 16:26'!
giveFocusToPreviousFrom: aMorph 

	^ self model giveFocusToPreviousFrom: aMorph ! !

!WindowModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/10/2012 16:21'!
initialExtent

	^ initialExtent contents
		ifNil: [ self model 
				ifNil: [ self defaultInitialExtent ]
				ifNotNil: [:m | m initialExtent
								ifNil: [ self defaultInitialExtent ]]]! !

!WindowModel methodsFor: 'initialize' stamp: 'BenjaminVanRyseghem 7/10/2012 15:29'!
initialize
	"Initialization code for WindowModel"

	super initialize.

	model := nil asValueHolder.
	initialExtent := nil asValueHolder.
	title := self class title asValueHolder.
	
	model whenChangedDo: [:m | self updateTitle ].
	title whenChangedDo: [ ]! !

!WindowModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/13/2012 01:49'!
isDisplayed

	^ self window
		ifNil: [ false ]
		ifNotNil: [:w | w isDisplayed ]! !

!WindowModel methodsFor: 'morphic' stamp: 'BenjaminVanRyseghem 7/10/2012 15:04'!
model
	
	^ model contents! !

!WindowModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 7/10/2012 15:17'!
model: aModel

	model contents: aModel! !

!WindowModel methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 8/2/2012 15:44'!
openModal: aWindow 

	self widget ifNotNil: [:w | w openModal: aWindow ]! !

!WindowModel methodsFor: 'private' stamp: 'BenjaminVanRyseghem 7/10/2012 15:19'!
openWithSpec: aSpec

	window := self buildWithSpec: aSpec.
	window openInWorld.
	window title: self title! !

!WindowModel methodsFor: 'private' stamp: 'BenjaminVanRyseghem 7/10/2012 15:43'!
openWithSpecLayout: aSpec

	window := self buildWithSpecLayout: aSpec.
	window openInWorld.
	window title: self title! !

!WindowModel methodsFor: 'private' stamp: 'BenjaminVanRyseghem 7/10/2012 15:42'!
rebuildWithSpec: aSpec
	| sub |
	
	sub := window submorphs copy.
	window removeAllMorphs.
	sub allButLast do: [:e | window addMorphBack: e ].
	self addModelIn: self widget withSpec: aSpec! !

!WindowModel methodsFor: 'private' stamp: 'BenjaminVanRyseghem 7/10/2012 15:43'!
rebuildWithSpecLayout: aSpec
	| sub |
	
	sub := window submorphs copy.
	window removeAllMorphs.
	sub allButLast do: [:e | window addMorphBack: e ].
	self addModelIn: self widget withSpecLayout: aSpec! !

!WindowModel methodsFor: 'private' stamp: 'BenjaminVanRyseghem 7/10/2012 15:18'!
title

	^ self model
		ifNil: [ super title ]
		ifNotNil: [:m | m title ]! !

!WindowModel methodsFor: 'private' stamp: 'BenjaminVanRyseghem 8/2/2012 17:14'!
title: aString

	^ self widget
		ifNotNil: [:w | w title: aString ]! !

!WindowModel methodsFor: 'private' stamp: 'BenjaminVanRyseghem 7/10/2012 15:36'!
updateTitle
	"Update the window title"

	self window
		ifNotNil: [:w | w title: self title ]! !

ComposableModel subclass: #MethodToolbar
	instanceVariableNames: 'methodHolder browseModel sendersModel implementorsModel versionModel dropListModel model'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets-PolyWidgets'!

!MethodToolbar commentStamp: '<historical>' prior: 0!
A MethodToolbar is xxxxxxxxx.

b := MethodToolbar new.
b openWithSpec.
b method: (ButtonModel>>#state:).
b method: nil!

!MethodToolbar class methodsFor: 'specs' stamp: 'bvr 6/4/2012 14:50'!
defaultSpec
	<spec>
	^ { #Panel.
			#changeTableLayout.
			#listDirection:.		#rightToLeft.
			#addMorph:. {#model.	#browseModel.}.
			#addMorph:. {#model.	#sendersModel.}.
			#addMorph:. {#model.	#implementorsModel.}.
			#addMorph:. {#model.	#versionModel.	}.
			#addMorph:. {#model.	#dropListModel.}.
			#hResizing:.	#spaceFill.
			#vResizing:.	#shrinkWrap.	}! !

!MethodToolbar class methodsFor: 'specs' stamp: ''!
title

	^ 'Toolbar'! !

!MethodToolbar methodsFor: 'private' stamp: 'StephaneDucasse 12/19/2012 16:55'!
addAll: aWindow  withSpec: aSpec

	aWindow
		addMorph: (self buildWithSpec: aSpec)
		fullFrame: ((0@0 corner: 1@0) asLayoutFrame bottomOffset: 30).
		! !

!MethodToolbar methodsFor: 'private' stamp: ''!
browseMethod

	methodHolder contents
		ifNotNil: [:aMethod | aMethod browse ]! !

!MethodToolbar methodsFor: 'accessing' stamp: ''!
browseModel

	^ browseModel! !

!MethodToolbar methodsFor: 'accessing' stamp: ''!
dropListModel

	^ dropListModel! !

!MethodToolbar methodsFor: 'private' stamp: ''!
implementorsMethod

	methodHolder contents
		ifNotNil: [:aMethod | self model browseMessagesFrom: aMethod selector ]! !

!MethodToolbar methodsFor: 'accessing' stamp: ''!
implementorsModel

	^ implementorsModel! !

!MethodToolbar methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/12/2012 18:00'!
initialize
	"Initialization code for MethodToolbar"

	methodHolder := nil asValueHolder.
	model := AbstractTool new.
	
	super initialize.! !

!MethodToolbar methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/12/2012 18:00'!
initializeWidgets
	
	self instantiateModels: #(
		browseModel  			ButtonModel
		sendersModel  			ButtonModel
		implementorsModel  	ButtonModel
		versionModel  			ButtonModel
		dropListModel  		DropListModel ).
				
	self 
		setFocusOrder;
		setBrowseModel;
		setVersionModel;
		setSendersModel;
		setImplementorsModel
! !

!MethodToolbar methodsFor: 'protocol' stamp: ''!
method: aMethod

	methodHolder contents: aMethod! !

!MethodToolbar methodsFor: 'accessing' stamp: ''!
model

	^ model! !

!MethodToolbar methodsFor: 'initialization' stamp: ''!
registerForChanges

	methodHolder whenChangedDo: [:contents || boolean |
		boolean := contents
						ifNil: [ false ]
						ifNotNil: [ true ].
		browseModel state: boolean.
		sendersModel state: boolean.
		versionModel state: boolean.
		implementorsModel state: boolean]! !

!MethodToolbar methodsFor: 'private' stamp: ''!
sendersMethod

	methodHolder contents
		ifNotNil: [:aMethod | self model browseSendersOfMessagesFrom: aMethod selector ]! !

!MethodToolbar methodsFor: 'accessing' stamp: ''!
sendersModel

	^ sendersModel! !

!MethodToolbar methodsFor: 'initialization' stamp: ''!
setBrowseModel
	
	browseModel
		state: false;
		label: 'Browse';
		action: [ self browseMethod ].
	
! !

!MethodToolbar methodsFor: 'protocol' stamp: ''!
setDropListItems: aCollection

	dropListModel items: aCollection! !

!MethodToolbar methodsFor: 'initialization' stamp: ''!
setFocusOrder

	self focusOrder
		add: browseModel;
		add: sendersModel;
		add: implementorsModel;
		add: versionModel.
		! !

!MethodToolbar methodsFor: 'initialization' stamp: ''!
setImplementorsModel
	
	implementorsModel
		state: false;
		label: 'Implementors';
		action: [ self implementorsMethod ].
	
! !

!MethodToolbar methodsFor: 'initialization' stamp: ''!
setSendersModel
	
	sendersModel
		state: false;
		label: 'Senders';
		action: [ self sendersMethod ].
	
! !

!MethodToolbar methodsFor: 'initialization' stamp: ''!
setVersionModel
	
	versionModel
		state: false;
		label: 'Version';
		action: [ self versionMethod ].
	
! !

!MethodToolbar methodsFor: 'private' stamp: ''!
versionMethod

	methodHolder contents
		ifNotNil: [:aMethod | self model browseVersionsFrom: aMethod ]! !

!MethodToolbar methodsFor: 'accessing' stamp: ''!
versionModel

	^ versionModel! !

ComposableModel subclass: #OkToolbar
	instanceVariableNames: 'okButton okAction'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets-PolyWidgets'!

!OkToolbar commentStamp: '<historical>' prior: 0!
An OkCancelToolbar is a model for a basic Ok toolbar!

OkToolbar subclass: #OkCancelToolbar
	instanceVariableNames: 'cancelButton cancelAction cancelled'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets-PolyWidgets'!

!OkCancelToolbar commentStamp: '<historical>' prior: 0!
An OkCancelToolbar is a model for a basic Ok-Cancel toolbar!

!OkCancelToolbar class methodsFor: 'specs' stamp: 'bvr 6/4/2012 14:51'!
defaultSpec
	<spec>
	^ { #Panel.
			#changeTableLayout.
			#listDirection:.		#rightToLeft.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#okButton.	}.
			#add:. {#model.	#cancelButton.}.
			#hSpaceFill.
			#vShrinkWrap.	}! !

!OkCancelToolbar class methodsFor: 'specs' stamp: ''!
title

	^ 'Ok Cancel'! !

!OkCancelToolbar methodsFor: 'protocol' stamp: ''!
cancelAction

	^ cancelAction contents! !

!OkCancelToolbar methodsFor: 'protocol' stamp: ''!
cancelAction: aBlock

	^ cancelAction contents: aBlock! !

!OkCancelToolbar methodsFor: 'accessing' stamp: ''!
cancelButton

	^ cancelButton! !

!OkCancelToolbar methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 5/11/2012 18:32'!
cancelled

	^ cancelled contents! !

!OkCancelToolbar methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/12/2012 18:03'!
initialize
	"Initialization code for OkCancelToolbar"

	cancelled := false asValueHolder.
	cancelAction := [ true ] asValueHolder.
	super initialize.! !

!OkCancelToolbar methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/12/2012 18:03'!
initializePresenter 

	super initializePresenter.
	cancelButton 
		action: [
			cancelAction contents value == false
					ifFalse: [
						cancelled contents: true.
						owner 
							ifNil: [ self delete ]
							ifNotNil: [ owner delete ]]].! !

!OkCancelToolbar methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/12/2012 18:07'!
initializeWidgets

	super initializeWidgets.
	self instantiateModels: #( cancelButton ButtonModel ).

	self registerCancelButtonEvents.
	self focusOrder
		add: cancelButton! !

!OkCancelToolbar methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/12/2012 18:03'!
registerCancelButtonEvents

	cancelButton 
		label: 'Cancel';
		enabled: true;
		state: false! !

!OkToolbar class methodsFor: 'specs' stamp: 'bvr 6/4/2012 14:51'!
defaultSpec
	<spec>
	^ { #Panel.
			#changeTableLayout.
			#listDirection:.		#rightToLeft.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#hFill}.
			#add:. {#model.	#okButton.}.
			#hSpaceFill.
			#vShrinkWrap.	}! !

!OkToolbar class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 5/10/2012 13:07'!
title

	^ 'Ok Toolbar'! !

!OkToolbar methodsFor: 'private' stamp: 'IgorStasenko 12/20/2012 14:03'!
addAll: aWindow  withSpec: aSpec

	aWindow
		addMorph: (self buildWithSpec: aSpec)
		fullFrame: ((0@0 corner: 1@0) asLayoutFrame bottomOffset: 30).! !

!OkToolbar methodsFor: 'private' stamp: 'BenjaminVanRyseghem 5/10/2012 13:05'!
hFill

	^ Morph new
		color: Color transparent;
		height: 0;
		vResizing: #rigid;
		hResizing: #spaceFill! !

!OkToolbar methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/11/2012 10:31'!
initialize
	"Initialization code for OkCancelToolbar"

	super initialize.
	okAction := [ true ] asValueHolder.! !

!OkToolbar methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/12/2012 18:01'!
initializePresenter 

	okButton 
		action: [ 
				okAction contents value == false
					ifFalse: [
						owner 
							ifNil: [ self delete ]
							ifNotNil: [ owner delete ]]].! !

!OkToolbar methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/12/2012 18:02'!
initializeWidgets
	
	
	self instantiateModels:  #( okButton ButtonModel ).
	self registerOkButtonEvents.
	
	self focusOrder
		add: okButton! !

!OkToolbar methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 5/10/2012 13:05'!
okAction

	^ okAction contents! !

!OkToolbar methodsFor: 'protocol' stamp: 'BenjaminVanRyseghem 5/10/2012 13:05'!
okAction: aBlock

	^ okAction contents: aBlock! !

!OkToolbar methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 5/10/2012 13:05'!
okButton

	^ okButton! !

!OkToolbar methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/12/2012 18:01'!
registerOkButtonEvents

	okButton 
		label: 'Ok';
		enabled: true;
		state: false! !

ComposableModel subclass: #PickListModel
	instanceVariableNames: 'defaultValueHolder labelClickableHolder listModel pickedItemsHolder wrapHolder associationsItemToWrapper associationsWrapperToItem'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets-PolyWidgets'!

!PickListModel commentStamp: '<historical>' prior: 0!
A PickList is a tick list done using spec.!

!PickListModel class methodsFor: 'specs' stamp: 'BenjaminVanRyseghem 6/12/2012 18:32'!
defaultSpec
	<spec>
	
	^ SpecLayout composed 
		add: #listModel;
		yourself! !

!PickListModel class methodsFor: 'example' stamp: ''!
example
	"self example"
	| instance |
	
	instance := self new.
	instance openWithSpec.
	instance items: { {1. 2. 4}. 'Foo'. 123}! !

!PickListModel class methodsFor: 'specs' stamp: ''!
title

	^ 'Pick List Example'! !

!PickListModel methodsFor: 'private' stamp: ''!
addPicked: item

	pickedItemsHolder contents add: item.
	pickedItemsHolder contentsChanged: true to: item.! !

!PickListModel methodsFor: 'protocol' stamp: ''!
checkboxFor: anItem

	^ associationsItemToWrapper at: anItem ifAbsent: [ nil ]! !

!PickListModel methodsFor: 'protocol' stamp: ''!
defaultValue

	^ defaultValueHolder contents! !

!PickListModel methodsFor: 'protocol' stamp: ''!
defaultValue: aBoolean

	defaultValueHolder contents: aBoolean! !

!PickListModel methodsFor: 'protocol' stamp: 'bvr 6/2/2012 12:44'!
displayBlock: aBlock

	wrapHolder contents: aBlock! !

!PickListModel methodsFor: 'initialization' stamp: 'BenjaminVanRyseghem 6/12/2012 18:05'!
initialize
	"Initialization code for PickList"

	wrapHolder := [ :i | i printString ] asValueHolder. 
	pickedItemsHolder := Set new asValueHolder.
	associationsWrapperToItem := Dictionary new asValueHolder.
	associationsItemToWrapper := Dictionary new asValueHolder.
	labelClickableHolder := true asValueHolder.
	defaultValueHolder := false asValueHolder.
	
	super initialize.! !

!PickListModel methodsFor: 'initialization' stamp: 'CamilloBruni 10/8/2012 22:23'!
initializePresenter
	
	labelClickableHolder whenChangedDo: [:aBoolean | 
		associationsWrapperToItem keys do: [:e | e labelClickable: aBoolean ]].

	defaultValueHolder whenChangedDo: [:value | associationsWrapperToItem keysDo: [:cb | cb state: value ]].! !

!PickListModel methodsFor: 'initialization' stamp: 'CamilloBruni 10/8/2012 22:15'!
initializeWidgets
	
	self instantiateModels: #( listModel ListComposableModel ).
	listModel displayBlock: [ :item | item ].
	self focusOrder add: listModel! !

!PickListModel methodsFor: 'protocol' stamp: ''!
itemFor: aCheckbox

	^ associationsWrapperToItem at: aCheckbox ifAbsent: [ nil ]! !

!PickListModel methodsFor: 'protocol' stamp: ''!
items: aCollection

	associationsWrapperToItem removeAll.
	pickedItemsHolder contents removeAll.
	pickedItemsHolder contentsChanged: false to: nil.
	listModel items: (aCollection collect: [:e | self wrap: e ]).! !

!PickListModel methodsFor: 'protocol' stamp: ''!
labelClickable
	
	^ labelClickableHolder contents! !

!PickListModel methodsFor: 'protocol' stamp: ''!
labelClickable: aBoolean

	labelClickableHolder contents: aBoolean! !

!PickListModel methodsFor: 'accessing' stamp: ''!
listModel

	^ listModel! !

!PickListModel methodsFor: 'protocol' stamp: ''!
pickedItems

	^ pickedItemsHolder contents! !

!PickListModel methodsFor: 'private' stamp: ''!
removePicked: item

	pickedItemsHolder contents remove: item ifAbsent: [ ^ self ].
	pickedItemsHolder contentsChanged: false to: item.! !

!PickListModel methodsFor: 'protocol' stamp: ''!
resetSelection

	listModel resetSelection! !

!PickListModel methodsFor: 'protocol' stamp: ''!
selectedIndex

	^ listModel selectedIndex! !

!PickListModel methodsFor: 'protocol' stamp: ''!
selectedItem

	^ associationsWrapperToItem 
		at: listModel selectedItem
		ifAbsent: [ nil ]! !

!PickListModel methodsFor: 'protocol' stamp: 'CamilloBruni 10/8/2012 21:56'!
setSelectedIndex: anIndex

	listModel setSelectedIndex: anIndex! !

!PickListModel methodsFor: 'protocol-events' stamp: 'bvr 5/31/2012 13:34'!
whenListChanged: aBlock

	listModel whenListChanged: aBlock! !

!PickListModel methodsFor: 'protocol-events' stamp: 'bvr 5/31/2012 13:34'!
whenPickedItemsChanged: aBlock

	pickedItemsHolder whenChangedDo: aBlock! !

!PickListModel methodsFor: 'protocol-events' stamp: 'bvr 5/31/2012 13:34'!
whenSelectedItemChanged: aBlock
	| newBlock |
	
	newBlock := [ :item | aBlock cull: (associationsWrapperToItem at: item ifAbsent: [ nil ])].
	listModel whenSelectedItemChanged: newBlock! !

!PickListModel methodsFor: 'protocol-events' stamp: 'StephaneDucasse 5/17/2012 19:32'!
whenSelectionChanged: aBlock

	listModel whenSelectionChanged: aBlock! !

!PickListModel methodsFor: 'protocol-events' stamp: 'bvr 5/31/2012 13:34'!
whenSelectionIndexChanged: aBlock

	listModel whenSelectionIndexChanged: aBlock! !

!PickListModel methodsFor: 'private' stamp: 'CamilloBruni 10/9/2012 13:55'!
wrap: item
	| checkBox |

	associationsItemToWrapper 
		at: item
		ifPresent: [:cb | ^ cb ].

	checkBox := self instantiate: CheckBoxModel.
	checkBox label: (wrapHolder contents cull: item cull: self).
	
	checkBox whenActivatedDo: [ 
		self addPicked: item.
		listModel setSelectedItem: checkBox.
		listModel takeKeyboardFocus ].
	
	checkBox whenDesactivatedDo: [ 
		self removePicked: item.
		listModel setSelectedItem: checkBox.
		listModel takeKeyboardFocus ].
	
	checkBox 
		state: self defaultValue;
		labelClickable: self labelClickable.
		
	associationsWrapperToItem at: checkBox put: item.
	associationsItemToWrapper at: item put: checkBox.
	^ checkBox! !

!PickListModel methodsFor: 'accessing' stamp: ''!
wrapHolder

	^ wrapHolder! !

ComposableModel subclass: #SearchableList
	instanceVariableNames: 'listModel searchModel'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Spec-Widgets-PolyWidgets'!

!SearchableList commentStamp: '<historical>' prior: 0!
SearchableList 
	new items: #( aaa abc axz bbb bfbg bof c);
	openWithSpec
	
should it be a subclass of LIstModel?
Because now I will have to delegate a lot? and I cannot plug a IconList

How do I make textModel react to every keystroke without an accept!

!SearchableList class methodsFor: 'specs' stamp: 'StephaneDucasse 12/21/2012 19:00'!
defaultSpec

	^ {#ComposableSpec .
		#add: . {{#model . #searchModel.}. #layout: . #(0 0  1 0 0 0 0 25) } .
		#add: . {{#model . #listModel.}. #layout: . #( 0 0 1 1 0 25 0 0 ) }
		}! !

!SearchableList methodsFor: 'initialize' stamp: 'BenjaminVanRyseghem 6/12/2012 18:06'!
initializePresenter

	searchModel whenTextChanged: [:newText || text |
		text :=  searchModel getText asLowercase.
		text isEmpty
			ifTrue: [ listModel resetFilteringBlock ]
			ifFalse: [
				listModel filteringBlock: [:col | col select: [:e | e asLowercase beginsWith: searchModel getText asLowercase ]]]]! !

!SearchableList methodsFor: 'initialize' stamp: 'BenjaminVanRyseghem 6/12/2012 18:06'!
initializeWidgets

	self instantiateModels: #(	
		listModel 		ListComposableModel
		searchModel 	TextInputFieldModel ).
	searchModel 
		autoAccept: true;
		ghostText: 'filter'.! !

!SearchableList methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 6/12/2012 18:46'!
listModel
	^ listModel! !

!SearchableList methodsFor: 'accessing' stamp: 'BenjaminVanRyseghem 6/12/2012 18:46'!
searchModel
	^ searchModel! !
